from typing import List, TypedDict


class InputTaskRequired(TypedDict):
    title: str


class InputTask(InputTaskRequired, total=False):
    group: str
    description: str
    status: str
    flag: bool
    startDate: int
    endDate: int
    order: int
    dueDate: int
    assignee: str
    mandatory: bool


class OutputTaskRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    title: str
    group: str
    status: str
    flag: bool
    order: int
    mandatory: bool
    extraData: dict


class OutputTask(OutputTaskRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    description: str
    startDate: int
    endDate: int
    assignee: str
    dueDate: int


class InputUpdateTask(TypedDict, total=False):
    title: str
    group: str
    description: str
    status: str
    flag: bool
    startDate: int
    endDate: int
    order: int
    dueDate: int
    assignee: str
    mandatory: bool


class InputBulkUpdateTask(InputUpdateTask):
    ids: List[str]



from typing import List, TypedDict


class InputTaskLogRequired(TypedDict):
    message: str


class InputTaskLog(InputTaskLogRequired, total=False):
    startDate: int
    includeInTimeline: int


class OutputTaskLogRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    message: str
    date: int
    owner: str
    extraData: dict


class OutputTaskLog(OutputTaskLogRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    attachments: List[dict]  # TODO: typehint
    includeInTimeline: int


class InputUpdateTaskLog(TypedDict, total=False):
    message: str
    includeInTimeline: int



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

original_alert_ids = []
for i in range(2):
    original_alert = hive.alert.create(
        alert={
            "type": "update-bulk",
            "source": "tutorial",
            "sourceRef": uuid.uuid4().hex,
            "title": f"original alert #{i}",
            "description": "an alert to update in bulk",
        }
    )

    original_alert_ids.append(original_alert["_id"])


hive.alert.bulk_update(
    fields={
        "ids": original_alert_ids,
        "title": "bulk updated alert",
        "tags": ["update-bulk"],
    },
)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.observable import (
    InputBulkUpdateObservable,
    InputObservable,
    InputUpdateObservable,
    OutputObservable,
)
from thehive4py.types.share import OutputShare


class ObservableEndpoint(EndpointBase):
    def create_in_alert(
        self,
        alert_id: str,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/observable", **kwargs
        )

    def create_in_case(
        self,
        case_id: str,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable", **kwargs
        )

    def get(self, observable_id: str) -> OutputObservable:
        return self._session.make_request(
            "GET", path=f"/api/v1/observable/{observable_id}"
        )

    def delete(self, observable_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/observable/{observable_id}"
        )

    def update(self, observable_id: str, fields: InputUpdateObservable) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/observable/{observable_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateObservable) -> None:
        return self._session.make_request(
            "PATCH", path="/api/v1/observable/_bulk", json=fields
        )

    def share(self, observable_id: str, organisations: List[str]) -> None:
        return self._session.make_request(
            "POST",
            path=f"/api/v1/observable/{observable_id}/shares",
            json={"organisations": organisations},
        )

    def unshare(self, observable_id: str, organisations: List[str]) -> None:
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/observable/{observable_id}/shares",
            json={"organisations": organisations},
        )

    def list_shares(self, observable_id: str) -> List[OutputShare]:
        return self._session.make_request(
            "GET", path=f"/api/v1/case/{observable_id}/shares"
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservable]:
        query: QueryExpr = [
            {"_name": "listObservable"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "observables"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        query: QueryExpr = [
            {"_name": "listObservable"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "observable.count"},
            json={"query": query},
        )

    def download_attachment(
        self,
        observable_id: str,
        attachment_id: str,
        observable_path: str,
        as_zip=False,
    ) -> None:
        return self._session.make_request(
            "GET",
            path=(
                f"/api/v1/observable/{observable_id}"
                f"/attachment/{attachment_id}/download"
            ),
            params={"asZip": as_zip},
            download_path=observable_path,
        )



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

case_to_get = hive.case.create(
    case={
        "title": "case to get",
        "description": "a single case to fetch",
    }
)


fetched_case = hive.case.get(case_id=case_to_get["_id"])



from typing import Optional

from requests import Response


class TheHiveError(Exception):

    def __init__(
        self, message: str, response: Optional[Response] = None, *args, **kwargs
    ):
        """Base error class of thehive4py.

        Args:
            message: The exception message.
            response: Either `None`, or a `Response` object of a failed request.
        """
        super().__init__(message, *args, **kwargs)
        self.message = message
        self.response = response



from typing import List

import pytest

from tests.utils import TestConfig, reset_hive_instance, spawn_hive_container
from thehive4py.client import TheHiveApi
from thehive4py.helpers import now_to_ts
from thehive4py.types.alert import InputAlert, OutputAlert
from thehive4py.types.case import InputCase, OutputCase
from thehive4py.types.case_template import InputCaseTemplate, OutputCaseTemplate
from thehive4py.types.comment import OutputComment
from thehive4py.types.custom_field import OutputCustomField
from thehive4py.types.observable import InputObservable, OutputObservable
from thehive4py.types.observable_type import OutputObservableType
from thehive4py.types.page import OutputCasePage
from thehive4py.types.procedure import OutputProcedure
from thehive4py.types.profile import OutputProfile
from thehive4py.types.task import InputTask, OutputTask
from thehive4py.types.task_log import InputTaskLog, OutputTaskLog
from thehive4py.types.timeline import OutputCustomEvent
from thehive4py.types.user import OutputUser


@pytest.fixture(scope="session")
def test_config():
    return TestConfig(
        image_name="strangebee/thehive:5.4.7",
        container_name="thehive4py-integration-tester",
        user="admin@thehive.local",
        password="secret",
        admin_org="admin",
        main_org="main-org",
        share_org="share-org",
    )


@pytest.fixture(scope="function", autouse=True)
def auto_reset_hive_instance(thehive: TheHiveApi, test_config: TestConfig):
    reset_hive_instance(hive_url=thehive.session.hive_url, test_config=test_config)


@pytest.fixture(scope="session")
def thehive(test_config: TestConfig):
    hive_url = spawn_hive_container(test_config=test_config)
    client = TheHiveApi(
        url=hive_url,
        username=test_config.user,
        password=test_config.password,
        organisation=test_config.main_org,
    )
    return client


@pytest.fixture
def thehive_admin(test_config: TestConfig, thehive: TheHiveApi):
    default_organisation = thehive.session_organisation
    thehive.session_organisation = test_config.admin_org
    yield thehive
    thehive.session_organisation = default_organisation


@pytest.fixture
def test_alert(thehive: TheHiveApi) -> OutputAlert:
    return thehive.alert.create(
        alert={
            "title": "my first alert",
            "description": "...",
            "type": "test",
            "source": "test",
            "sourceRef": "first",
            "externalLink": "http://",
            "date": now_to_ts(),
            "tags": ["whatever"],
        }
    )


@pytest.fixture
def test_alerts(thehive: TheHiveApi) -> List[OutputAlert]:
    alerts: List[InputAlert] = [
        {
            "title": "my first alert",
            "description": "...",
            "type": "test",
            "source": "test",
            "sourceRef": "first",
            "date": now_to_ts(),
        },
        {
            "title": "my second alert",
            "description": "...",
            "type": "test",
            "source": "test",
            "sourceRef": "second",
            "date": now_to_ts(),
        },
    ]
    return [thehive.alert.create(alert=alert) for alert in alerts]


@pytest.fixture
def test_case(thehive: TheHiveApi) -> OutputCase:
    return thehive.case.create(
        case={"title": "my first case", "description": "...", "tags": ["whatever"]}
    )


@pytest.fixture
def test_cases(thehive: TheHiveApi) -> List[OutputCase]:
    cases: List[InputCase] = [
        {"title": "my first case", "description": "...", "tags": ["whatever"]},
        {"title": "my second case", "description": "...", "tags": ["whatever"]},
    ]
    return [thehive.case.create(case=case) for case in cases]


@pytest.fixture
def test_case_template(thehive: TheHiveApi) -> OutputCaseTemplate:
    name = "my first case template"
    return thehive.case_template.create(
        case_template={
            "name": name,
            "description": "...",
            "tags": ["template-tag"],
        }
    )


@pytest.fixture
def test_case_templates(thehive: TheHiveApi) -> List[OutputCaseTemplate]:
    case_templates: List[InputCaseTemplate] = [
        {
            "name": "my first case template",
            "description": "...",
            "tags": ["template-tag-1"],
        },
        {
            "name": "my second case template",
            "description": "...",
            "tags": ["template-tag-2"],
        },
    ]
    return [
        thehive.case_template.create(case_template=case_template)
        for case_template in case_templates
    ]


@pytest.fixture
def test_observable(thehive: TheHiveApi, test_case: OutputCase) -> OutputObservable:
    return thehive.observable.create_in_case(
        case_id=test_case["_id"],
        observable={
            "dataType": "domain",
            "data": "example.com",
            "message": "test observable",
            "tlp": 1,
            "pap": 1,
            "tags": ["test-tag"],
        },
    )[0]


@pytest.fixture
def test_observables(
    thehive: TheHiveApi, test_case: OutputCase
) -> List[OutputObservable]:
    observables: List[InputObservable] = [
        {
            "dataType": "domain",
            "data": "example.com",
            "message": "test observable",
            "tlp": 1,
            "pap": 1,
            "tags": ["test-tag"],
        },
        {
            "dataType": "ip",
            "data": "127.0.0.1",
            "message": "test observable",
            "tlp": 2,
            "pap": 2,
            "tags": ["test-tag"],
        },
    ]
    return [
        thehive.observable.create_in_case(
            case_id=test_case["_id"], observable=observable
        )[0]
        for observable in observables
    ]


@pytest.fixture
def test_task(thehive: TheHiveApi, test_case: OutputCase) -> OutputTask:
    return thehive.task.create(
        case_id=test_case["_id"],
        task={"title": "my first task"},
    )


@pytest.fixture
def test_tasks(thehive: TheHiveApi, test_case: OutputCase) -> List[OutputTask]:
    tasks: List[InputTask] = [
        {"title": "my first task"},
        {"title": "my second task"},
    ]
    return [thehive.task.create(case_id=test_case["_id"], task=task) for task in tasks]


@pytest.fixture
def test_task_log(thehive: TheHiveApi, test_task: OutputTask) -> OutputTaskLog:
    return thehive.task_log.create(
        task_id=test_task["_id"],
        task_log={"message": "test log"},
    )


@pytest.fixture
def test_task_logs(thehive: TheHiveApi, test_task: OutputTask) -> List[OutputTaskLog]:
    task_logs: List[InputTaskLog] = [
        {"message": "my first log"},
        {"message": "my second log"},
    ]
    return [
        thehive.task_log.create(task_id=test_task["_id"], task_log=task_log)
        for task_log in task_logs
    ]


@pytest.fixture
def test_comment(thehive: TheHiveApi, test_case: OutputCase) -> OutputComment:
    return thehive.comment.create_in_case(
        case_id=test_case["_id"],
        comment={"message": "my first comment"},
    )


@pytest.fixture
def test_procedure(thehive: TheHiveApi, test_case: OutputCase) -> OutputProcedure:
    return thehive.procedure.create_in_case(
        case_id=test_case["_id"],
        procedure={
            "occurDate": now_to_ts(),
            "patternId": "T1059.006",
            "tactic": "execution",
            "description": "...",
        },
    )


@pytest.fixture
def test_case_page(thehive: TheHiveApi, test_case: OutputCase) -> OutputCasePage:
    return thehive.case.create_page(
        case_id=test_case["_id"],
        page={
            "title": "my case page",
            "category": "testing",
            "content": "...",
        },
    )


@pytest.fixture
def test_timeline_event(
    thehive: TheHiveApi, test_case: OutputCase
) -> OutputCustomEvent:
    return thehive.timeline.create_event(
        case_id=test_case["_id"],
        event={
            "date": now_to_ts(),
            "endDate": now_to_ts(),
            "title": "test timeline event",
            "description": "...",
        },
    )


@pytest.fixture
def test_user(test_config: TestConfig, thehive: TheHiveApi) -> OutputUser:
    return thehive.user.create(
        user={
            "email": "user@example.com",
            "name": "test user",
            "login": "user@example.com",
            "profile": "analyst",
            "organisation": test_config.main_org,
        }
    )


@pytest.fixture
def test_profile(thehive_admin: TheHiveApi) -> OutputProfile:
    return thehive_admin.profile.create(
        profile={"name": "my-read-only", "permissions": []}
    )


@pytest.fixture
def test_custom_field(thehive_admin: TheHiveApi) -> OutputCustomField:
    return thehive_admin.custom_field.create(
        custom_field={
            "name": "test-field",
            "type": "string",
            "displayName": "Test Field",
            "description": "...",
            "group": "default",
            "options": [],
        }
    )


@pytest.fixture
def test_observable_type(thehive_admin: TheHiveApi) -> OutputObservableType:
    return thehive_admin.observable_type.create(
        observable_type={"name": "my-observable-type"}
    )



from thehive4py import TheHiveApi
from thehive4py.types.alert import InputAlert

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

input_alert: InputAlert = {
    "type": "advanced",
    "source": "tutorial",
    "sourceRef": "should-be-unique",
    "title": "an advanced alert",
    "description": "a bit more advanced",
    "tags": ["advanced", "example"],
    "severity": 1,
    "caseTemplate": "my-template",
}

output_alert = hive.alert.create(alert=input_alert)



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

original_case = hive.case.create(
    case={
        "title": "original case",
        "description": "a single case to update",
    }
)


hive.case.update(
    case_id=original_case["_id"],
    fields={
        "title": "updated case",
        "tags": ["update-single"],
    },
)

updated_case = hive.case.get(case_id=original_case["_id"])



from typing import Any, Dict, List, TypedDict


class OutputAnalyzerRequired(TypedDict):
    id: str
    name: str
    version: str
    description: str


class OutputAnalyzer(OutputAnalyzerRequired, total=False):
    dataTypeList: List[str]
    cortexIds: List[str]


class OutputResponderRequired(TypedDict):
    id: str
    name: str
    version: str
    description: str


class OutputResponder(OutputResponderRequired, total=False):
    dataTypeList: List[str]
    cortexIds: List[str]


class OutputAnalyzerJobRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: str
    analyzerId: str
    analyzerName: str
    analyzerDefinition: str
    status: str
    startDate: str
    cortexId: str
    cortexJobId: str
    id: str
    operations: str


class OutputAnalyzerJob(OutputAnalyzerJobRequired, total=False):
    _updatedBy: str
    _updatedAt: str
    endDate: str
    report: Dict[str, Any]
    case_artifact: Dict[str, Any]


class OutputResponderActionRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: str
    responderId: str
    status: str
    startDate: str
    cortexId: str
    cortexJobId: str
    id: str
    operations: str


class OutputResponderAction(OutputResponderActionRequired, total=False):
    _updatedBy: str
    _updatedAt: str
    endDate: str
    report: Dict[str, Any]
    responderName: str
    responderDefinition: str


class InputResponderActionRequired(TypedDict):
    objectId: str
    objectType: str
    responderId: str


class InputResponderAction(InputResponderActionRequired, total=False):
    parameters: Dict[str, Any]
    tlp: int


class InputAnalyzerJobRequired(TypedDict):
    analyzerId: str
    cortexId: str
    artifactId: str


class InputAnalyzerJob(InputAnalyzerJobRequired, total=False):
    parameters: Dict[str, Any]



import json as jsonlib
import warnings
from typing import List, Optional, Sequence, Union

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.attachment import OutputAttachment
from thehive4py.types.case import (
    CaseStatus,
    CaseStatusValue,
    ImpactStatusValue,
    InputApplyCaseTemplate,
    InputBulkUpdateCase,
    InputCase,
    InputImportCase,
    InputUpdateCase,
    OutputCase,
)
from thehive4py.types.comment import OutputComment
from thehive4py.types.observable import InputObservable, OutputObservable
from thehive4py.types.page import InputCasePage, InputUpdateCasePage, OutputCasePage
from thehive4py.types.procedure import InputProcedure, OutputProcedure
from thehive4py.types.share import InputShare, OutputShare
from thehive4py.types.task import InputTask, OutputTask
from thehive4py.types.timeline import OutputTimeline

CaseId = Union[str, int]


class CaseEndpoint(EndpointBase):
    def create(self, case: InputCase) -> OutputCase:
        """Create a case.

        Args:
            case: The body of the case.

        Returns:
            The created case.
        """
        return self._session.make_request("POST", path="/api/v1/case", json=case)

    def get(self, case_id: CaseId) -> OutputCase:
        """Get a case by id.

        Args:
            case_id: The id of the case.

        Returns:
            The case specified by the id.
        """
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}")

    def delete(self, case_id: CaseId) -> None:
        """Delete a case.

        Args:
            case_id: The id of the case.

        Returns:
            N/A
        """
        self._session.make_request("DELETE", path=f"/api/v1/case/{case_id}")

    def update(
        self, case_id: CaseId, fields: Optional[InputUpdateCase] = {}, **kwargs
    ) -> None:
        """Update a case.

        Args:
            case_id: The id of the case.
            fields: The fields of the case to update.

        Returns:
            N/A
        """

        if not fields:
            if "case" not in kwargs:
                raise TheHiveError(
                    f"Unrecognized keyword arguments: {list(kwargs.keys())}. "
                    "Please use the `fields` argument to supply case update values."
                )
            warnings.warn(
                message="The `case` argument has been deprecated to follow the same "
                "convention like other update methods. Please use the `fields` "
                "argument to prevent breaking changes in the future.",
                category=DeprecationWarning,
                stacklevel=2,
            )
            fields = kwargs["case"]

        return self._session.make_request(
            "PATCH", path=f"/api/v1/case/{case_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateCase) -> None:
        """Update multiple cases with the same values.

        Args:
            fields: The ids and the fields of the cases to update.

        Returns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path="/api/v1/case/_bulk", json=fields
        )

    def merge(self, case_ids: Sequence[CaseId]) -> OutputCase:
        """Merge multiple cases into one final case.

        Args:
            case_ids: The ids of the cases to merge.

        Returns:
            The merged case.
        """
        case_id_subpath = ",".join([str(case_id) for case_id in case_ids])
        return self._session.make_request(
            "POST", path=f"/api/v1/case/_merge/{case_id_subpath}"
        )

    def unlink_alert(self, case_id: str, alert_id: str) -> None:
        """Unlink an alert from a case.

        Args:
            case_id: The id of the case to unlink the alert from.
            alert_id: The id of the alert to unlink.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/{case_id}/alert/{alert_id}"
        )

    def merge_similar_observables(self, case_id: CaseId) -> dict:
        """Merge similar observables of a case.

        Args:
            case_id: The id of the case to merge similar observables for.

        Returns:
            The metadata of the observable merge operation.
        """
        # TODO: add better return value type hint
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable/_merge"
        )

    def get_linked_cases(self, case_id: CaseId) -> List[OutputCase]:
        """Get other cases linked to a case.

        Args:
            case_id: The id of the case to get linked cases for.

        Returns:
            The list of linked cases.
        """
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}/links")

    def delete_custom_field(self, custom_field_id: str) -> None:
        """Delete a custom field from a case.

        Args:
            custom_field_id: The id of the specific custom field to delete from a case.

        Retruns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/customField/{custom_field_id}"
        )

    def import_from_file(self, import_case: InputImportCase, import_path: str) -> dict:
        """Import a case from a .thar archive file.

        Args:
            import_case: The metadata of the case import.
            import_path: The filepath to the .thar archive.

        Returns:
            The metadata of the case import operation.
        """
        # TODO: add better return type hints
        return self._session.make_request(
            "POST",
            path="/api/v1/case/import",
            data={"_json": jsonlib.dumps(import_case)},
            files={"file": self._fileinfo_from_filepath(import_path)},
        )

    def export_to_file(self, case_id: CaseId, password: str, export_path: str) -> None:
        """Export a case to a .thar archive file.

        The file can be used to import the case in an other TheHive instance

        Args:
            case_id: The id of the case to export.
            password: The password to encrypt the .thar file with.
            export_path: The filepath to save the case export to.

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET",
            path=f"/api/v1/case/{case_id}/export",
            params={"password": password},
            download_path=export_path,
        )

    def get_timeline(self, case_id: CaseId) -> OutputTimeline:
        """Get the timeline of a case.

        Args:
            case_id: The id of the case with the timeline.

        Returns:
            The case timeline.
        """
        return self._session.make_request("GET", f"/api/v1/case/{case_id}/timeline")

    def apply_case_template(self, fields: InputApplyCaseTemplate) -> None:
        """Retroactively apply a case template on a case.

        Args:
            fields: The metadata of the case template apply operation.

        Returns:
            N/A
        """
        return self._session.make_request(
            "POST", "/api/v1/case/_bulk/caseTemplate", json=fields
        )

    def add_attachment(
        self, case_id: CaseId, attachment_paths: List[str]
    ) -> List[OutputAttachment]:
        """Create an attachment in a case.

        Args:
            case_id: The id of the case.
            attachment_paths: List of paths to the attachments to create.

        Returns:
            The created case attachments.
        """

        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", f"/api/v1/case/{case_id}/attachments", files=files
        )["attachments"]

    def download_attachment(
        self, case_id: CaseId, attachment_id: str, attachment_path: str
    ) -> None:
        """Download a case attachment.

        Args:
            case_id: The id of the case.
            attachment_id: The id of the case attachment.
            attachment_path: The local path to download the attachment to.

        Returns:
            N/A
        """
        return self._session.make_request(
            "GET",
            path=f"/api/v1/case/{case_id}/attachment/{attachment_id}/download",
            download_path=attachment_path,
        )

    def delete_attachment(self, case_id: CaseId, attachment_id: str) -> None:
        """Delete a case attachment.

        Args:
            case_id: The id of the case.
            attachment_id: The id of the case attachment.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/{case_id}/attachment/{attachment_id}"
        )

    def list_shares(self, case_id: CaseId) -> List[OutputShare]:
        """List all organisation shares of a case.

        Args:
            case_id: The id of the case.

        Returns:
            The list of organisation shares of the case.
        """
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}/shares")

    def share(self, case_id: CaseId, shares: List[InputShare]) -> List[OutputShare]:
        """Share the case with other organisations.

        For each organisation, you can define a profile (level of access) that the org
        will receive. This request will only create new shares and will not update or
        delete existing shares.

        Args:
            case_id: The id of the case.
            shares: The list of organisational share rules.

        Returns:
            The list of organisation shares of the case.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/shares", json={"shares": shares}
        )

    def unshare(self, case_id: CaseId, organisation_ids: List[str]) -> None:
        """Unshare a case from other organisations.

        Args:
            case_id: The id of the case.
            organisation_ids: The ids of the organisations to unshare from.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/case/{case_id}/shares",
            json={"organisations": organisation_ids},
        )

    def set_share(self, case_id: CaseId, shares: List[InputShare]) -> List[OutputShare]:
        """Set the share for a case with other organisations.

        For each organisation, you can define a profile (level of access) that the org
        will receive. This request can delete and update already existing shares.

        Args:
            case_id: The id of the case.
            shares: The list of organisational share rules.

        Returns:
            The list of organisation shares of the case.
        """
        return self._session.make_request(
            "PUT", path=f"/api/v1/case/{case_id}/shares", json={"shares": shares}
        )

    def remove_share(self, share_id: str) -> None:
        """Remove a specific organisation share from a case.

        Args:
            share_id: The id of the share to remove.

        Returns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/share/{share_id}"
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputCase]:
        """Find multiple cases.

        Args:
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of cases matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "listCase"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        """Count cases.

        Args:
            filters: The filter expressions to apply in the query.

        Returns:
            The count of cases matched by the query.
        """
        query: QueryExpr = [
            {"_name": "listCase"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases.count"},
            json={"query": query},
        )

    def create_task(self, case_id: CaseId, task: InputTask) -> OutputTask:
        """Create a case task.

        Args:
            case_id: The id of the case.
            task: The fields of the task to create.

        Returns:
            The created case task.
        """
        return self._session.make_request(
            "POST",
            path=f"/api/v1/case/{case_id}/task",
            json=task,
        )

    def find_tasks(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTask]:
        """Find tasks related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case tasks matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "tasks"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-tasks"},
            json={"query": query},
        )

    def create_observable(
        self,
        case_id: CaseId,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        """Create an observable in an case.

        Args:
            case_id: The id of the case.
            observable: The fields of the observable to create.
            observable_path: Optional path in case of a file based observable.

        Returns:
            The created case observables.
        """
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable", **kwargs
        )

    def find_observables(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservable]:
        """Find observables related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case observables matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "observables"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-observables"},
            json={"query": query},
        )

    def create_procedure(
        self, case_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        """Create a case procedure.

        Args:
            case_id: The id of the case.
            procedure: The fields of the procedure to create.

        Returns:
            The created case procedure.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/procedure", json=procedure
        )

    def find_procedures(
        self,
        case_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        """Find procedures related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case procedures matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "procedures"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-procedures"},
            json={"query": query},
        )

    def create_page(self, case_id: str, page: InputCasePage) -> OutputCasePage:
        """Create a page in a case.

        Args:
            case_id: The id of the case.
            page: The fields of the page to create.

        Returns:
            The created case page.
        """
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/page", json=page
        )

    def delete_page(self, case_id: str, page_id: str) -> None:
        """Delete a page from a case.

        Args:
            case_id: The id of the case.
            page_id: The id of the page to delete.

        Retruns:
            N/A
        """
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/{case_id}/page/{page_id}"
        )

    def update_page(
        self, case_id: str, page_id: str, page: InputUpdateCasePage
    ) -> None:
        """Update a page of a case.

        Args:
            case_id: The id of the case.
            page_id: The id of the page to update.
            page: The fields of the page to update.

        Retruns:
            N/A
        """
        return self._session.make_request(
            "PATCH", path=f"/api/v1/case/{case_id}/page/{page_id}", json=page
        )

    def find_pages(
        self,
        case_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        """Find pages related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case pages matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "pages"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-pages"},
            json={"query": query},
        )

    def find_attachments(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputAttachment]:
        """Find attachments related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case attachments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "attachments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-attachments"},
            json={"query": query},
        )

    def find_comments(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputComment]:
        """Find comments related to a case.

        Args:
            case_id: The id of the case.
            filters: The filter expressions to apply in the query.
            sortby: The sort expressions to apply in the query.
            paginate: The pagination experssion to apply in the query.

        Returns:
            The list of case comments matched by the query or an empty list.
        """
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "comments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-comments"},
            json={"query": query},
        )

    def close(
        self,
        case_id: CaseId,
        status: CaseStatusValue,
        summary: str,
        impact_status: ImpactStatusValue = "NotApplicable",
    ) -> None:
        """Close a case.

        Args:
            case_id: The id of the case.
            status: The status to close the case with.
            summary: The closure summary of the case.
            impact_status: The impact status of the case.

        Returns:
            N/A
        """
        case: InputUpdateCase = {
            "status": status,
            "impactStatus": impact_status,
            "summary": summary,
        }
        return self.update(
            case_id,
            case,
        )

    def open(
        self, case_id: CaseId, status: CaseStatusValue = CaseStatus.InProgress
    ) -> None:
        """Open a closed case.

        Args:
            case_id: The id of the case.
            status: The status to re-open the case with.

        Returns:
            N/A
        """
        case: InputUpdateCase = {"status": status}
        return self.update(case_id, case)



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

alert_ids_to_delete = []

for i in range(2):
    alert_to_delete = hive.alert.create(
        alert={
            "type": "delete",
            "source": "tutorial",
            "sourceRef": uuid.uuid4().hex,
            "title": f"delete alert #{i}",
            "description": "an alert to delete",
        }
    )
    alert_ids_to_delete.append(alert_to_delete["_id"])


hive.alert.bulk_delete(ids=alert_ids_to_delete)



import typing

from thehive4py.endpoints._base import EndpointBase


class QueryEndpoint(EndpointBase):
    def run(
        self, query: typing.List[dict], exclude_fields: typing.List[str] = []
    ) -> typing.Any:
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query, "excludeFields": exclude_fields},
        )



#!/usr/bin/env python
import argparse
import re


def replace_pr_reference(match: re.Match) -> str:
    pr_number = match.groups()[0]
    pr_link = (
        f" [#{pr_number}]"
        "(https://github.com/TheHive-Project/TheHive4py/pull/{pr_number})"
    )
    print(f"Replace `{match.group()}` by `{pr_link}`")
    return pr_link


def linkify_pr_references(release_notes: str) -> str:
    pr_ref_pattern = r" #(\d+)"
    linkified_release_notes = re.sub(
        pattern=pr_ref_pattern,
        repl=replace_pr_reference,
        string=release_notes,
    )
    return linkified_release_notes


def replace_pr_url(match: re.Match) -> str:
    pr_url, pr_number = match.groups()
    pr_link = f" [#{pr_number}]({pr_url})"
    print(f"Replace `{match.group()}` by `{pr_link}`")
    return pr_link


def linkify_pr_urls(release_notes: str) -> str:
    pr_url_pattern = (
        r" (https:\/\/github\.com\/TheHive-Project\/TheHive4py\/pull\/(\d+))"
    )

    linkified_release_notes = re.sub(
        pattern=pr_url_pattern,
        repl=replace_pr_url,
        string=release_notes,
    )
    return linkified_release_notes


def replace_contributor_reference(match: re.Match) -> str:
    contributor = match.groups()[0]
    contributor_link = f" [@{contributor}](https://github.com/{contributor})"
    print(f"Replace `{match.group()}` by `{contributor_link}`")
    return contributor_link


def linkify_contributor_references(release_notes: str) -> str:
    contributor_ref_pattern = r" @(\w+)"

    linkified_release_notes = re.sub(
        pattern=contributor_ref_pattern,
        repl=replace_contributor_reference,
        string=release_notes,
    )
    return linkified_release_notes


def replace_full_changelog_url(match: re.Match) -> str:
    changelog_url, changelog_tags = match.groups()
    full_changelog_link = f" [{changelog_tags}]({changelog_url})"
    print(f"Replace `{match.group()}` by `{full_changelog_link}`")
    return full_changelog_link


def linkify_full_changelog_urls(release_notes: str) -> str:
    full_changelog_url_pattern = (
        r" (https:\/\/github\.com/TheHive-Project\/TheHive4py\/compare\/(.+\.\.\..+))"
    )

    linkified_release_notes = re.sub(
        pattern=full_changelog_url_pattern,
        repl=replace_full_changelog_url,
        string=release_notes,
    )
    return linkified_release_notes


def parse_arguments():
    parser = argparse.ArgumentParser(
        prog="linkify-release-notes",
        description=("enhance release notes with markdown links"),
    )
    parser.add_argument(
        "-c",
        "--check",
        action="store_true",
        default=False,
        help="just check for link replacements",
    )

    return parser.parse_args()


def main():

    args = parse_arguments()

    release_notes_path = "docs/release-notes.md"
    with open(release_notes_path) as release_notes_fp:
        release_notes = release_notes_fp.read()

    print(f"Checking linkification in '{release_notes_path}'")
    linkified_release_notes = release_notes
    linkified_release_notes = linkify_pr_references(
        release_notes=linkified_release_notes
    )
    linkified_release_notes = linkify_pr_urls(release_notes=linkified_release_notes)
    linkified_release_notes = linkify_contributor_references(
        release_notes=linkified_release_notes
    )
    linkified_release_notes = linkify_full_changelog_urls(
        release_notes=linkified_release_notes
    )

    if linkified_release_notes == release_notes:
        print("Nothing to do, release notes are already linkified!")
        return

    if args.check:
        print("The `--check` flag is active, exiting without replacing!")
        exit(1)

    with open(release_notes_path, "w") as linkified_release_notes_fp:
        linkified_release_notes_fp.write(linkified_release_notes)

    print(f"Successfully linkified '{release_notes_path}'!")


if __name__ == "__main__":
    main()



from typing import List

import pytest

from tests.utils import TestConfig
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.query.filters import Eq
from thehive4py.types.user import InputUpdateUser, InputUserOrganisation, OutputUser


class TestUserEndpoint:
    def test_get_current(self, thehive: TheHiveApi):
        # TODO: implement a better control for user checking
        current_user = thehive.user.get_current()
        assert current_user["login"] == "admin@thehive.local"

    def test_create_and_get(self, thehive: TheHiveApi):
        created_user = thehive.user.create(
            user={"login": "bailando@sin.ti", "name": "Bailando", "profile": "analyst"}
        )
        created_user_2 = thehive.user.create(
            user={"login": "bailando@sin.ti", "name": "Bailando", "profile": "analyst"}
        )

        assert created_user == created_user_2

        fetched_user = thehive.user.get(created_user["_id"])
        assert created_user == fetched_user

    def test_update(
        self, test_config: TestConfig, thehive: TheHiveApi, test_user: OutputUser
    ):
        user_id = test_user["_id"]
        update_fields: InputUpdateUser = {
            "name": "Updated user",
            "profile": "read-only",
            "email": "whatever@example.com",
            "organisation": test_config.main_org,
        }
        thehive.user.update(user_id=user_id, fields=update_fields)
        updated_user = thehive.user.get(user_id=user_id)

        for field in update_fields:
            assert updated_user.get(field) == update_fields.get(field)

    def test_lock_and_unlock(self, thehive: TheHiveApi, test_user: OutputUser):
        user_id = test_user["_id"]

        thehive.user.lock(user_id=user_id)
        locked_user = thehive.user.get(user_id=user_id)
        assert locked_user["locked"] is True

        thehive.user.unlock(user_id=user_id)
        unlocked_user = thehive.user.get(user_id=user_id)
        assert unlocked_user["locked"] is False

    def test_delete(self, thehive: TheHiveApi, test_user: OutputUser):
        user_id = test_user["_id"]
        thehive.user.delete(user_id=user_id, organisation=test_user["organisation"])
        with pytest.raises(TheHiveError):
            thehive.user.get(user_id=user_id)

    @pytest.mark.skip(reason="integrator container only supports a single org ")
    def test_set_organisations(
        self, test_config: TestConfig, thehive: TheHiveApi, test_user: OutputUser
    ):
        organisations: List[InputUserOrganisation] = [
            {
                "default": True,
                "organisation": test_user["organisation"],
                "profile": "analyst",
            },
            {
                "default": False,
                "organisation": test_config.share_org,
                "profile": "read-only",
            },
        ]
        user_organisations = thehive.user.set_organisations(
            user_id=test_user["_id"], organisations=organisations
        )
        assert organisations == user_organisations

    def test_set_password(self, thehive: TheHiveApi, test_user: OutputUser):
        assert test_user["hasPassword"] is False
        user_id = test_user["_id"]

        password = "super-secruht!"
        thehive.user.set_password(user_id=user_id, password=password)

        user_with_password = thehive.user.get(user_id=user_id)
        assert user_with_password["hasPassword"] is True

    def test_renew_get_and_remove_apikey(
        self, thehive: TheHiveApi, test_user: OutputUser
    ):
        assert test_user["hasKey"] is False
        user_id = test_user["_id"]

        thehive.user.renew_apikey(user_id=user_id)
        assert isinstance(thehive.user.get_apikey(user_id=user_id), str)
        user_with_apikey = thehive.user.get(user_id=user_id)
        assert user_with_apikey["hasKey"] is True

        thehive.user.remove_apikey(user_id=user_id)
        user_without_apikey = thehive.user.get(user_id=user_id)
        assert user_without_apikey["hasKey"] is False

    def test_find_and_count(self, thehive: TheHiveApi, test_user: OutputUser):
        user_filters = Eq("login", test_user["login"])
        found_users = thehive.user.find(filters=user_filters)
        user_count = thehive.user.count(filters=user_filters)

        assert [test_user] == found_users
        assert len(found_users) == user_count



from thehive4py.client import TheHiveApi


def _warn_old_py_version(min_py_version=(3, 9)):
    import sys
    import warnings

    if sys.version_info < min_py_version:
        warnings.warn(
            message=(
                "thehive4py will drop support for Python versions below "
                f"{min_py_version[0]}.{min_py_version[1]}, as they have reached their "
                "end of life. Please upgrade to a newer version as soon as possible."
            ),
            category=DeprecationWarning,
            stacklevel=3,
        )


_warn_old_py_version()



from typing import List, TypedDict


class OutputAttachmentRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    name: str
    size: int
    contentType: str
    id: str


class OutputAttachment(OutputAttachmentRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    hashes: List[str]



from typing import TypedDict


class InputObservableTypeRequired(TypedDict):
    name: str


class InputObservableType(InputObservableTypeRequired, total=False):
    isAttachment: bool


class OutputObservableTypeRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    name: str
    isAttachment: bool


class OutputObservableType(OutputObservableTypeRequired, total=False):
    _updatedBy: str
    _updatedAt: int



from thehive4py import TheHiveApi
from thehive4py.types.observable import InputObservable

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")


case_to_enrich = hive.case.create(
    case={
        "title": "case to enrich",
        "description": "a case to enrich with simple observables",
    }
)


observables: list[InputObservable] = [
    {"dataType": "ip", "data": "1.2.3.4"},
    {"dataType": "domain", "data": "example.com"},
]


for observable in observables:
    hive.case.create_observable(case_id=case_to_enrich["_id"], observable=observable)



import uuid

from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

alert_to_get = hive.alert.create(
    alert={
        "type": "get-single",
        "source": "tutorial",
        "sourceRef": uuid.uuid4().hex,
        "title": "alert to get",
        "description": "a single alert to fetch",
    }
)


fetched_alert = hive.alert.get(alert_id=alert_to_get["_id"])



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.task import (
    InputBulkUpdateTask,
    InputTask,
    InputUpdateTask,
    OutputTask,
)
from thehive4py.types.task_log import InputTaskLog, OutputTaskLog


class TaskEndpoint(EndpointBase):
    def create(self, case_id: str, task: InputTask) -> OutputTask:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/task", json=task
        )

    def get(self, task_id: str) -> OutputTask:
        return self._session.make_request("GET", path=f"/api/v1/task/{task_id}")

    def delete(self, task_id: str) -> None:
        return self._session.make_request("DELETE", path=f"/api/v1/task/{task_id}")

    def update(self, task_id: str, fields: InputUpdateTask) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/task/{task_id}", json=fields
        )

    def bulk_update(self, fields: InputBulkUpdateTask) -> None:
        return self._session.make_request(
            "PATCH", path="/api/v1/task/_bulk", json=fields
        )

    def get_required_actions(self, task_id: str) -> dict:
        return self._session.make_request(
            "GET", path=f"/api/v1/task/{task_id}/actionRequired"
        )

    def set_as_required(self, task_id: str, org_id: str) -> None:
        return self._session.make_request(
            "PUT", f"/api/v1/task/{task_id}/actionRequired/{org_id}"
        )

    def set_as_done(self, task_id: str, org_id: str) -> None:
        return self._session.make_request(
            "PUT", f"/api/v1/task/{task_id}/actionDone/{org_id}"
        )

    def share(self):
        raise NotImplementedError()

    def list_shares(self):
        raise NotImplementedError()

    def unshare(self):
        raise NotImplementedError()

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTask]:
        query: QueryExpr = [
            {"_name": "listTask"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "tasks"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        query: QueryExpr = [
            {"_name": "listTask"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "task.count"},
            json={"query": query},
        )

    def create_log(self, task_id: str, task_log: InputTaskLog) -> OutputTaskLog:
        return self._session.make_request(
            "POST", path=f"/api/v1/task/{task_id}/log", json=task_log
        )

    def find_logs(
        self,
        task_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTaskLog]:
        query: QueryExpr = [
            {"_name": "getTask", "idOrName": task_id},
            {"_name": "logs"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-task-logs"},
            json={"query": query},
        )



from typing import List

from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.custom_field import (
    InputCustomField,
    InputUpdateCustomField,
    OutputCustomField,
)


class CustomFieldEndpoint(EndpointBase):
    def create(self, custom_field: InputCustomField) -> OutputCustomField:
        return self._session.make_request(
            "POST", path="/api/v1/customField", json=custom_field
        )

    def list(self) -> List[OutputCustomField]:
        return self._session.make_request("GET", path="/api/v1/customField")

    def delete(self, custom_field_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/customField/{custom_field_id}"
        )

    def update(self, custom_field_id: str, fields: InputUpdateCustomField) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/customField/{custom_field_id}", json=fields
        )



from thehive4py.client import TheHiveApi
from thehive4py.types.case import OutputCase
from thehive4py.types.observable import OutputObservable


class TestCortexEndpoint:
    def test_create_analyzer_job(
        self, thehive: TheHiveApi, test_observable: OutputObservable
    ):
        output_analyzer_job = thehive.cortex.create_analyzer_job(
            job={
                "analyzerId": "example",
                "cortexId": "cortex",
                "artifactId": test_observable["_id"],
            }
        )
        assert output_analyzer_job["_type"] == "case_artifact_job"

    def test_list_analyzers(self, thehive: TheHiveApi):
        analyzers = thehive.cortex.list_analyzers()
        assert analyzers == []

    def test_list_analyzers_by_type(self, thehive: TheHiveApi):
        data_type = "mail"
        analyzers = thehive.cortex.list_analyzers_by_type(data_type=data_type)
        assert analyzers == []

    def test_list_responders(self, thehive: TheHiveApi, test_case: OutputCase):
        responders = thehive.cortex.list_responders(
            entity_type="case", entity_id=test_case["_id"]
        )
        assert responders == []



import pytest
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.query.filters import Eq
from thehive4py.types.observable_type import OutputObservableType


class TestObservableTypeEndpoint:
    def test_create_and_get(self, thehive_admin: TheHiveApi):
        created_observable_type = thehive_admin.observable_type.create(
            {
                "name": "new-observable-type",
            }
        )

        test_observable_type = thehive_admin.observable_type.get(
            created_observable_type["_id"]
        )
        assert created_observable_type == test_observable_type

    def test_find(
        self, thehive: TheHiveApi, test_observable_type: OutputObservableType
    ):
        found_observable_types = thehive.observable_type.find(
            filters=Eq("name", test_observable_type["name"])
        )

        assert found_observable_types == [test_observable_type]

    def test_delete(
        self, thehive: TheHiveApi, test_observable_type: OutputObservableType
    ):
        observable_type_id = test_observable_type["_id"]
        thehive.observable_type.delete(observable_type_id)
        with pytest.raises(TheHiveError):
            thehive.observable_type.get(observable_type_id)



import pytest

from thehive4py import TheHiveApi
from thehive4py.helpers import now_to_ts
from thehive4py.query.filters import (
    Between,
    Contains,
    EndsWith,
    Eq,
    Gt,
    Gte,
    Has,
    Id,
    In,
    Like,
    Lt,
    Lte,
    Match,
    Ne,
    StartsWith,
)
from thehive4py.types.user import OutputUser


class TestQueryFilters:
    def test_between_contains_has_in(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(
            filters=Between(field="_createdAt", start=0, end=now_to_ts())
        )
        with pytest.deprecated_call():
            assert thehive.user.find(filters=Contains(field="login"))
        assert thehive.user.find(filters=Has(field="login"))
        assert thehive.user.find(
            filters=In(field="login", values=["...", "xyz", test_user["login"]])
        )

    def test_endswith_startswith(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(
            filters=EndsWith(field="login", value=test_user["login"])
        )
        assert thehive.user.find(
            filters=StartsWith(field="login", value=test_user["login"])
        )

    def test_eq_ne_id(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(filters=Eq(field="_id", value=test_user["_id"]))
        assert thehive.user.find(filters=Ne(field="_id", value=test_user["login"]))
        assert thehive.user.find(filters=Id(id=test_user["_id"]))

    def test_gt_gte_lt_lte(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(filters=Gt(field="_createdAt", value=0))
        assert thehive.user.find(filters=Gte(field="_createdAt", value=0))
        assert thehive.user.find(filters=Lt(field="_createdAt", value=now_to_ts()))
        assert thehive.user.find(filters=Lte(field="_createdAt", value=now_to_ts()))

    def test_like_match(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(filters=Like(field="login", value=test_user["login"]))
        assert thehive.user.find(filters=Match(field="login", value=test_user["login"]))

    def test_and_or_not(self, thehive: TheHiveApi, test_user: OutputUser):
        assert thehive.user.find(
            filters=Id(id=test_user["_id"])
            & (
                Eq(field="login", value=test_user["login"])
                | Eq(field="login", value="...")
            )
            & ~Eq(field="login", value="...")
        )



from urllib3 import Retry

from thehive4py import TheHiveApi

simple_retries = Retry(
    total=5,
    backoff_factor=0.5,
    allowed_methods=["GET"],
    status_forcelist=[500],
)

hive = TheHiveApi(
    url="http://localhost:9000",
    apikey="h1v3b33",
    max_retries=simple_retries,
)



from typing import List, TypedDict


class InputUserRequired(TypedDict):
    login: str
    name: str
    profile: str


class InputUser(InputUserRequired, total=False):
    email: str
    password: str
    organisation: str
    type: str


class OutputOrganisationProfile(TypedDict):
    organisationId: str
    organisation: str
    profile: str


class OutputUserRequired(TypedDict):
    _id: str
    _createdBy: str
    _createdAt: int
    login: str
    name: str
    hasKey: bool
    hasPassword: bool
    hasMFA: bool
    locked: bool
    profile: str
    organisation: str
    type: str
    extraData: dict


class OutputUser(OutputUserRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    email: str
    permissions: List[str]
    avatar: str
    organisations: List[OutputOrganisationProfile]
    defaultOrganisation: str


class InputUpdateUser(TypedDict, total=False):
    name: str
    organisation: str
    profile: str
    locked: bool
    avatar: str
    email: str
    defaultOrganisation: str


class InputUserOrganisationRequired(TypedDict):
    organisation: str
    profile: str


class InputUserOrganisation(InputUserOrganisationRequired, total=False):
    default: bool


class OutputUserOrganisation(TypedDict):
    organisation: str
    profile: str
    default: bool



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.user import (
    InputUpdateUser,
    InputUser,
    InputUserOrganisation,
    OutputUser,
    OutputUserOrganisation,
)


class UserEndpoint(EndpointBase):
    def create(self, user: InputUser) -> OutputUser:
        return self._session.make_request("POST", path="/api/v1/user", json=user)

    def get(self, user_id: str) -> OutputUser:
        return self._session.make_request("GET", path=f"/api/v1/user/{user_id}")

    def get_current(self) -> OutputUser:
        return self._session.make_request("GET", path="/api/v1/user/current")

    def delete(self, user_id: str, organisation: Optional[str] = None) -> None:
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/user/{user_id}/force",
            params={"organisation": organisation},
        )

    def update(self, user_id: str, fields: InputUpdateUser) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/user/{user_id}", json=fields
        )

    def lock(self, user_id: str) -> None:
        return self.update(user_id=user_id, fields={"locked": True})

    def unlock(self, user_id: str) -> None:
        return self.update(user_id=user_id, fields={"locked": False})

    def set_organisations(
        self, user_id: str, organisations: List[InputUserOrganisation]
    ) -> List[OutputUserOrganisation]:
        return self._session.make_request(
            "PUT",
            path=f"/api/v1/user/{user_id}/organisations",
            json={"organisations": organisations},
        )["organisations"]

    def set_password(self, user_id: str, password: str) -> None:
        return self._session.make_request(
            "POST",
            path=f"/api/v1/user/{user_id}/password/set",
            json={"password": password},
        )

    def get_apikey(self, user_id: str) -> str:
        return self._session.make_request("GET", path=f"/api/v1/user/{user_id}/key")

    def remove_apikey(self, user_id: str) -> None:
        return self._session.make_request("DELETE", path=f"/api/v1/user/{user_id}/key")

    def renew_apikey(self, user_id: str) -> str:
        return self._session.make_request(
            "POST", path=f"/api/v1/user/{user_id}/key/renew"
        )

    def get_avatar(self, user_id: str):
        # TODO: implement the avatar download
        raise NotImplementedError()

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputUser]:
        query: QueryExpr = [
            {"_name": "listUser"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "users"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        query: QueryExpr = [
            {"_name": "listUser"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases.count"},
            json={"query": query},
        )



from typing import List, TypedDict


class InputProfileRequired(TypedDict):
    name: str


class InputProfile(InputProfileRequired, total=False):
    permissions: List[str]


class OutputProfileRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    name: str
    editable: bool
    isAdmin: bool


class OutputProfile(OutputProfileRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    permissions: List[str]


class InputUpdateProfile(TypedDict, total=False):
    name: str
    permissions: List[str]



from typing import List, Literal, TypedDict

from thehive4py.types.page import InputCasePage
from thehive4py.types.share import InputShare

from .custom_field import InputCustomFieldValue, OutputCustomFieldValue
from .task import InputTask

CaseStatusValue = Literal[
    "New",
    "InProgress",
    "Indeterminate",
    "FalsePositive",
    "TruePositive",
    "Other",
    "Duplicated",
]


class CaseStatus:
    New: CaseStatusValue = "New"
    InProgress: CaseStatusValue = "InProgress"
    Indeterminate: CaseStatusValue = "Indeterminate"
    FalsePositive: CaseStatusValue = "FalsePositive"
    TruePositive: CaseStatusValue = "TruePositive"
    Other: CaseStatusValue = "Other"
    Duplicated: CaseStatusValue = "Duplicated"


ImpactStatusValue = Literal["NotApplicable", "WithImpact", "NoImpact"]


class ImpactStatus:
    NotApplicable: ImpactStatusValue = "NotApplicable"
    WithImpact: ImpactStatusValue = "WithImpact"
    NoImpact: ImpactStatusValue = "NoImpact"


class InputCaseRequired(TypedDict):
    title: str
    description: str


class InputCase(InputCaseRequired, total=False):
    severity: int
    startDate: int
    endDate: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    status: CaseStatusValue
    summary: str
    assignee: str
    customFields: List[InputCustomFieldValue]
    caseTemplate: str
    tasks: List[InputTask]
    pages: List[InputCasePage]
    sharingParameters: List[InputShare]
    taskRule: str
    observableRule: str


class OutputCaseRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    number: int
    title: str
    description: str
    severity: int
    severityLabel: str
    startDate: int
    flag: bool
    tlp: int
    tlpLabel: str
    pap: int
    papLabel: str
    status: CaseStatusValue
    stage: str
    extraData: dict
    newDate: int
    timeToDetect: int


class OutputCase(OutputCaseRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    endDate: int
    tags: List[str]
    summary: str
    impactStatus: ImpactStatusValue
    assignee: str
    customFields: List[OutputCustomFieldValue]
    userPermissions: List[str]
    inProgressDate: int
    closedDate: int
    alertDate: int
    alertNewDate: int
    alertInProgressDate: int
    alertImportedDate: int
    timeToTriage: int
    timeToQualify: int
    timeToAcknowledge: int
    timeToResolve: int
    handlingDuration: int


class InputUpdateCase(TypedDict, total=False):
    title: str
    description: str
    severity: int
    startDate: int
    endDate: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    status: str
    summary: str
    assignee: str
    impactStatus: str
    customFields: List[InputCustomFieldValue]
    taskRule: str
    observableRule: str
    addTags: List[str]
    removeTags: List[str]


class InputBulkUpdateCase(InputUpdateCase):
    ids: List[str]


class InputImportCaseRequired(TypedDict):
    password: str


class InputImportCase(InputImportCaseRequired, total=False):
    sharingParameters: List[InputShare]
    taskRule: str
    observableRule: str


class InputApplyCaseTemplateRequired(TypedDict):
    ids: List[str]
    caseTemplate: str


class InputApplyCaseTemplate(InputApplyCaseTemplateRequired, total=False):
    updateTitlePrefix: bool
    updateDescription: bool
    updateTags: bool
    updateSeverity: bool
    updateFlag: bool
    updateTlp: bool
    updatePap: bool
    updateCustomFields: bool
    importTasks: List[str]
    importPages: List[str]



from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.case_template import OutputCaseTemplate, InputCaseTemplate
from typing import List, Optional


class CaseTemplateEndpoint(EndpointBase):
    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputCaseTemplate]:
        query: QueryExpr = [
            {"_name": "listCaseTemplate"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": query},
            params={"name": "caseTemplate"},
        )

    def get(self, case_template_id: str) -> OutputCaseTemplate:
        return self._session.make_request(
            "GET", path=f"/api/v1/caseTemplate/{case_template_id}"
        )

    def create(self, case_template: InputCaseTemplate) -> OutputCaseTemplate:
        return self._session.make_request(
            "POST", path="/api/v1/caseTemplate", json=case_template
        )

    def delete(self, case_template_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/caseTemplate/{case_template_id}"
        )

    def update(self, case_template_id: str, fields: InputCaseTemplate) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/caseTemplate/{case_template_id}", json=fields
        )



import pytest
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.types.alert import OutputAlert
from thehive4py.types.case import OutputCase
from thehive4py.types.comment import InputUpdateComment, OutputComment


class TestCommentEndpoint:
    def test_create_in_alert_and_get(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        created_comment = thehive.comment.create_in_alert(
            alert_id=test_alert["_id"],
            comment={
                "message": "test comment",
            },
        )

        fetched_comment = thehive.comment.get(comment_id=created_comment["_id"])
        assert created_comment == fetched_comment

    def test_create_in_case_and_get(self, thehive: TheHiveApi, test_case: OutputCase):
        created_comment = thehive.comment.create_in_case(
            case_id=test_case["_id"],
            comment={
                "message": "test comment",
            },
        )

        fetched_comment = thehive.comment.get(comment_id=created_comment["_id"])
        assert created_comment == fetched_comment

    def test_delete(self, thehive: TheHiveApi, test_comment: OutputComment):
        comment_id = test_comment["_id"]
        thehive.comment.delete(comment_id=comment_id)
        with pytest.raises(TheHiveError):
            thehive.comment.get(comment_id=comment_id)

    def test_update(self, thehive: TheHiveApi, test_comment: OutputComment):
        comment_id = test_comment["_id"]
        update_fields: InputUpdateComment = {
            "message": "updated comment",
        }
        thehive.comment.update(comment_id=test_comment["_id"], fields=update_fields)
        updated_comment = thehive.comment.get(comment_id=comment_id)

        for key, value in update_fields.items():
            assert updated_comment.get(key) == value



from thehive4py import TheHiveApi
from thehive4py.query.filters import Eq

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

antivirus_case = hive.case.create(
    case={
        "title": "case #1 to find",
        "description": "a case to find with others",
        "tags": ["antivirus"],
    }
)

phishing_case = hive.case.create(
    case={
        "title": "case #2 to find",
        "description": "a case to find with others",
        "tags": ["phishing"],
    }
)


filters = Eq(field="tags", value="antivirus") | Eq(field="tags", value="phishing")
fetched_cases = hive.case.find(filters=filters)



from collections import UserDict


class SortExpr(UserDict):
    """Base class for sort expressions."""

    def __and__(self, other: "SortExpr") -> "SortExpr":
        return self._concat_expressions("&", self, other)

    def __or__(self, other: "SortExpr") -> "SortExpr":  # type:ignore
        return self._concat_expressions("|", self, other)

    def _concat_expressions(
        self, operand: str, expr1: "SortExpr", expr2: "SortExpr"
    ) -> "SortExpr":
        if not isinstance(expr1, SortExpr) or not isinstance(expr2, SortExpr):
            self._raise_type_error(operand, expr1, expr2)
        return SortExpr(_fields=[*expr1["_fields"], *expr2["_fields"]])

    def _raise_type_error(self, operand, first, second):
        raise TypeError(
            f"unsupported operand type(s) for {operand}: "
            f"{type(first)} and {type(second)}"
        )


class Asc(SortExpr):
    def __init__(self, field: str):
        super().__init__(_fields=[{field: "asc"}])


class Desc(SortExpr):
    def __init__(self, field: str):
        super().__init__(_fields=[{field: "desc"}])



from typing import List, TypedDict

from thehive4py.types.custom_field import InputCustomFieldValue, OutputCustomFieldValue
from thehive4py.types.observable import InputObservable
from thehive4py.types.page import InputCasePage
from thehive4py.types.procedure import InputProcedure
from thehive4py.types.share import InputShare
from thehive4py.types.task import InputTask


class InputAlertRequired(TypedDict):
    type: str
    source: str
    sourceRef: str
    title: str
    description: str


class InputAlert(InputAlertRequired, total=False):
    date: int
    externalLink: str
    severity: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    customFields: List[InputCustomFieldValue]
    summary: str
    status: str
    assignee: str
    caseTemplate: str
    observables: List[InputObservable]
    procedures: List[InputProcedure]


class OutputAlertRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    type: str
    source: str
    sourceRef: str
    title: str
    description: str
    severity: int
    severityLabel: str
    date: int
    tlp: int
    tlpLabel: str
    pap: int
    papLabel: str
    follow: bool
    observableCount: int
    status: str
    stage: str
    extraData: dict
    newDate: int
    timeToDetect: int


class OutputAlert(OutputAlertRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    externalLink: str
    tags: List[str]
    customFields: List[OutputCustomFieldValue]
    caseTemplate: str
    caseId: str
    assignee: str
    summary: str
    inProgressDate: int
    closedDate: int
    importedDate: int
    timeToTriage: int
    timeToQualify: int
    timeToAcknowledge: int


class InputUpdateAlert(TypedDict, total=False):
    type: str
    source: str
    sourceRef: str
    externalLink: str
    title: str
    description: str
    severity: int
    date: int
    lastSyncDate: int
    tags: List[str]
    tlp: int
    pap: int
    follow: bool
    customFields: List[InputCustomFieldValue]
    status: str
    summary: str
    assignee: str
    addTags: List[str]
    removeTags: List[str]


class InputBulkUpdateAlert(InputUpdateAlert):
    ids: List[str]


class InputPromoteAlert(TypedDict, total=False):
    title: str
    description: str
    severity: int
    startDate: int
    endDate: int
    tags: List[str]
    flag: bool
    tlp: int
    pap: int
    status: str
    summary: str
    assignee: str
    customFields: List[InputCustomFieldValue]
    caseTemplate: str
    tasks: List[InputTask]
    pages: List[InputCasePage]
    sharingParameters: List[InputShare]
    taskRule: str
    observableRule: str



from .alert import AlertEndpoint
from .case import CaseEndpoint
from .case_template import CaseTemplateEndpoint
from .comment import CommentEndpoint
from .cortex import CortexEndpoint
from .custom_field import CustomFieldEndpoint
from .observable import ObservableEndpoint
from .organisation import OrganisationEndpoint
from .procedure import ProcedureEndpoint
from .profile import ProfileEndpoint
from .task import TaskEndpoint
from .task_log import TaskLogEndpoint
from .timeline import TimelineEndpoint
from .user import UserEndpoint



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.procedure import (
    InputProcedure,
    InputUpdateProcedure,
    OutputProcedure,
)


class ProcedureEndpoint(EndpointBase):
    def create_in_alert(
        self, alert_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/procedure", json=procedure
        )

    def create_in_case(
        self, case_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/procedure", json=procedure
        )

    def get(self, procedure_id: str) -> OutputProcedure:
        # TODO: temp implementation until a dedicated get endpoint
        procedures = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getProcedure", "idOrName": procedure_id}]},
        )
        try:
            return procedures[0]
        except IndexError:
            raise TheHiveError("404 - Procedure not found")

    def delete(self, procedure_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/procedure/{procedure_id}"
        )

    def update(self, procedure_id: str, fields: InputUpdateProcedure) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/procedure/{procedure_id}", json=fields
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        query: QueryExpr = [
            {"_name": "listProcedure"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "procedures"},
            json={"query": query},
        )



from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.timeline import (
    InputCustomEvent,
    InputUpdateCustomEvent,
    OutputCustomEvent,
    OutputTimeline,
)


class TimelineEndpoint(EndpointBase):
    def get(self, case_id: str) -> OutputTimeline:
        return self._session.make_request("GET", f"/api/v1/case/{case_id}/timeline")

    def create_event(self, case_id: str, event: InputCustomEvent) -> OutputCustomEvent:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/customEvent", json=event
        )

    def delete_event(self, event_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/customEvent/{event_id}"
        )

    def update_event(self, event_id: str, fields: InputUpdateCustomEvent) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/customEvent/{event_id}", json=fields
        )



from thehive4py import TheHiveApi

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

case_ids_to_merge: list[str] = []
for i in range(2):
    case_to_merge = hive.case.create(
        case={
            "title": f"original case #{i}",
            "description": "a case to merge with another",
        }
    )
    case_ids_to_merge.append(case_to_merge["_id"])


merge_case = hive.case.merge(case_ids=case_ids_to_merge)



from typing import List, TypedDict


class OutputTimelineEventRequired(TypedDict):
    date: int
    kind: str
    entity: str
    entityId: str
    details: dict


class OutputTimelineEvent(OutputTimelineEventRequired, total=False):
    endDate: int


class OutputTimeline(TypedDict):
    events: List[OutputTimelineEvent]


class InputCustomEventRequired(TypedDict):
    date: int
    title: str


class InputCustomEvent(InputCustomEventRequired, total=False):
    endDate: int
    description: str


class OutputCustomEventRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    date: int
    title: str


class OutputCustomEvent(OutputCustomEventRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    endDate: int
    description: str


class InputUpdateCustomEvent(TypedDict, total=False):
    date: int
    endDate: int
    title: str
    description: str



from pathlib import Path
from typing import List

import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.helpers import now_to_ts
from thehive4py.query.filters import Eq
from thehive4py.query.sort import Asc
from thehive4py.types.alert import InputBulkUpdateAlert, InputUpdateAlert, OutputAlert
from thehive4py.types.case import OutputCase
from thehive4py.types.observable import InputObservable


class TestAlertEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi):
        created_alert = thehive.alert.create(
            {
                "title": "my first alert",
                "description": "...",
                "type": "test",
                "source": "test",
                "sourceRef": "first",
                "externalLink": "http://",
                "date": 123,
                "tags": ["whatever"],
            }
        )

        fetched_alert = thehive.alert.get(created_alert["_id"])
        assert created_alert == fetched_alert

    def test_update(self, thehive: TheHiveApi, test_alert: OutputAlert):
        alert_id = test_alert["_id"]
        update_fields: InputUpdateAlert = {
            "title": "my updated alert",
            "description": "my updated description",
        }
        thehive.alert.update(alert_id=alert_id, fields=update_fields)
        updated_alert = thehive.alert.get(alert_id=alert_id)

        assert updated_alert["title"] == update_fields["title"]
        assert updated_alert["description"] == update_fields["description"]

    def test_bulk_update(self, thehive: TheHiveApi, test_alerts: List[OutputAlert]):
        alert_ids = [alert["_id"] for alert in test_alerts]
        update_fields: InputBulkUpdateAlert = {
            "ids": alert_ids,
            "title": "my updated alert #1",
            "description": "my updated description #1",
        }

        thehive.alert.bulk_update(fields=update_fields)
        updated_alerts = thehive.alert.find()

        for updated_alert in updated_alerts:
            assert updated_alert["title"] == update_fields["title"]
            assert updated_alert["description"] == update_fields["description"]

    def test_follow_and_unfollow(self, thehive: TheHiveApi, test_alert: OutputAlert):
        alert_id = test_alert["_id"]

        thehive.alert.unfollow(alert_id=alert_id)
        unfollowed_alert = thehive.alert.get(alert_id=alert_id)
        assert unfollowed_alert["follow"] is False

        thehive.alert.follow(alert_id=alert_id)
        followed_alert = thehive.alert.get(alert_id=alert_id)
        assert followed_alert["follow"] is True

    def test_promote_to_case(self, thehive: TheHiveApi, test_alert: OutputAlert):
        alert_id = test_alert["_id"]

        case_from_alert = thehive.alert.promote_to_case(
            alert_id=alert_id, fields={"title": "promoted title"}
        )
        promoted_alert = thehive.alert.get(alert_id=alert_id)
        assert promoted_alert.get("caseId") == case_from_alert["_id"]
        assert promoted_alert["title"] != case_from_alert["title"]

    def test_merge_into_case(
        self, thehive: TheHiveApi, test_alert: OutputAlert, test_case: OutputCase
    ):
        alert_id = test_alert["_id"]
        case_id = test_case["_id"]

        merged_case = thehive.alert.merge_into_case(alert_id=alert_id, case_id=case_id)

        merged_alert = thehive.alert.get(alert_id=alert_id)
        assert merged_alert.get("caseId") == merged_case["_id"]

    def test_bulk_merge_into_case(
        self, thehive: TheHiveApi, test_alerts: List[OutputAlert], test_case: OutputCase
    ):
        alert_ids = [alert["_id"] for alert in test_alerts]
        case_id = test_case["_id"]

        merged_case = thehive.alert.bulk_merge_into_case(
            case_id=case_id, alert_ids=alert_ids
        )

        for alert in thehive.alert.find():
            assert alert.get("caseId") == merged_case["_id"]

    def test_find_and_count(self, thehive: TheHiveApi, test_alerts: List[OutputAlert]):
        found_alerts = thehive.alert.find(
            filters=Eq("title", test_alerts[0]["title"])
            | Eq("title", test_alerts[1]["title"]),
            sortby=Asc("_createdAt"),
        )
        alert_count = thehive.alert.count()

        assert found_alerts == test_alerts
        assert len(found_alerts) == alert_count

    def test_delete(self, thehive: TheHiveApi, test_alert: OutputAlert):
        alert_id = test_alert["_id"]
        thehive.alert.delete(alert_id)
        with pytest.raises(TheHiveError):
            thehive.alert.get(alert_id)

    def test_bulk_delete(self, thehive: TheHiveApi, test_alerts: List[OutputAlert]):
        alert_ids = [alert["_id"] for alert in test_alerts]
        thehive.alert.bulk_delete(ids=alert_ids)
        for alert_id in alert_ids:
            with pytest.raises(TheHiveError):
                thehive.alert.get(alert_id)

    def test_create_and_get_observable(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        created_observables = thehive.alert.create_observable(
            test_alert["_id"], {"dataType": "domain", "data": "example.com"}
        )
        alert_observables = thehive.alert.find_observables(test_alert["_id"])
        assert created_observables == alert_observables

    def test_create_observable_from_file(
        self, thehive: TheHiveApi, test_alert: OutputAlert, tmp_path: Path
    ):
        observable_path = str(tmp_path / "alert-observable.txt")
        with open(observable_path, "w") as observable_fp:
            observable_fp.write("observable content")

        created_observable = thehive.observable.create_in_alert(
            alert_id=test_alert["_id"],
            observable={
                "dataType": "file",
                "message": "file based observable",
            },
            observable_path=observable_path,
        )[0]

        fetched_observable = thehive.observable.get(
            observable_id=created_observable["_id"]
        )
        assert created_observable == fetched_observable

        attachment = fetched_observable.get("attachment")
        assert attachment and attachment["name"] in observable_path

    def test_create_alert_with_observables(self, thehive: TheHiveApi):
        created_alert = thehive.alert.create(
            {
                "title": "my first alert",
                "description": "...",
                "type": "test",
                "source": "test",
                "sourceRef": "second",
                "externalLink": "http://",
                "date": 123,
                "tags": ["whatever"],
                "observables": [
                    {"dataType": "url", "data": "example.org"},
                    {"dataType": "mail", "data": "foo@example.org"},
                ],
            }
        )

        fetched_alert = thehive.alert.get(created_alert["_id"])
        # TODO: for some reason alert creation doesn't return _updatedAt and _updatedBy
        # ask backend team  what's the matter
        fetched_alert.pop("_updatedAt")
        fetched_alert.pop("_updatedBy")
        assert created_alert == fetched_alert
        assert created_alert["observableCount"] == 2

    def test_create_alert_with_observable_file(
        self, thehive: TheHiveApi, tmp_path: Path
    ):
        attachment_path = str(tmp_path / "alert-observable.txt")
        with open(attachment_path, "w") as attachment_fp:
            attachment_fp.write("observable content")

        alert_observables: List[InputObservable] = [
            {"dataType": "url", "data": "example.com"},
            {"dataType": "file", "attachment": "obs1"},
        ]
        created_alert = thehive.alert.create(
            alert={
                "title": "my first alert",
                "description": "...",
                "type": "test",
                "source": "test",
                "sourceRef": "third",
                "date": 123,
                "observables": alert_observables,
            },
            attachment_map={"obs1": attachment_path},
        )
        assert created_alert["observableCount"] == len(alert_observables)

    def test_find_comments(self, thehive: TheHiveApi, test_alert: OutputAlert):
        created_comment = thehive.comment.create_in_alert(
            alert_id=test_alert["_id"],
            comment={"message": "my first comment"},
        )

        alert_comments = thehive.alert.find_comments(alert_id=test_alert["_id"])

        assert [created_comment] == alert_comments

    def test_create_and_find_procedure(
        self, thehive: TheHiveApi, test_alert: OutputAlert
    ):
        created_procedure = thehive.alert.create_procedure(
            alert_id=test_alert["_id"],
            procedure={
                "occurDate": now_to_ts(),
                "patternId": "T1059.006",
                "tactic": "execution",
                "description": "...",
            },
        )
        alert_procedures = thehive.alert.find_procedures(alert_id=test_alert["_id"])
        assert [created_procedure] == alert_procedures

    def test_add_and_download_attachment(
        self, thehive: TheHiveApi, test_alert: OutputAlert, tmp_path: Path
    ):
        attachment_paths = [str(tmp_path / f"attachment-{i}.txt") for i in range(2)]
        download_attachment_paths = [
            str(tmp_path / f"dl-attachment-{i}.txt") for i in range(2)
        ]

        for path in attachment_paths:
            with open(path, "w") as attachment_fp:
                attachment_fp.write(f"content of {path}")

        added_attachments = thehive.alert.add_attachment(
            alert_id=test_alert["_id"], attachment_paths=attachment_paths
        )

        for attachment, path in zip(added_attachments, download_attachment_paths):
            thehive.alert.download_attachment(
                alert_id=test_alert["_id"],
                attachment_id=attachment["_id"],
                attachment_path=path,
            )

        for original, downloaded in zip(attachment_paths, download_attachment_paths):
            with open(original) as original_fp, open(downloaded) as downloaded_fp:
                assert original_fp.read() == downloaded_fp.read()

    def test_add_and_delete_attachment(
        self, thehive: TheHiveApi, test_alert: OutputAlert, tmp_path: Path
    ):
        attachment_path = str(tmp_path / "my-attachment.txt")
        with open(attachment_path, "w") as attachment_fp:
            attachment_fp.write("some content...")

        added_attachments = thehive.alert.add_attachment(
            alert_id=test_alert["_id"], attachment_paths=[attachment_path]
        )

        for attachment in added_attachments:
            thehive.alert.delete_attachment(
                alert_id=test_alert["_id"], attachment_id=attachment["_id"]
            )

        assert thehive.alert.find_attachments(alert_id=test_alert["_id"]) == []



from typing import List, TypedDict


class InputOrganisationLink(TypedDict, total=False):
    linkType: str
    otherLinkType: str


class InputBulkOrganisationLink(TypedDict):
    toOrganisation: str
    linkType: str
    otherLinkType: str


class OutputSharingProfile(TypedDict):
    name: str
    description: str
    autoShare: bool
    editable: bool
    permissionProfile: str
    taskRule: str
    observableRule: str


class InputOrganisationRequired(TypedDict):
    name: str
    description: str


class InputOrganisation(InputOrganisationRequired, total=False):
    taskRule: str
    observableRule: str
    locked: bool


class OutputOrganisationRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    name: str
    description: str
    taskRule: str
    observableRule: str
    locked: bool
    extraData: dict


class OutputOrganisation(OutputOrganisationRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    links: List[InputOrganisationLink]
    avatar: str


class InputUpdateOrganisation(TypedDict, total=False):
    name: str
    description: str
    taskRule: str
    observableRule: str
    locked: bool
    avatar: str



from thehive4py.endpoints._base import EndpointBase
from thehive4py.types.cortex import (
    OutputAnalyzer,
    OutputAnalyzerJob,
    OutputResponder,
    OutputResponderAction,
    InputResponderAction,
    InputAnalyzerJob,
)
from typing import Optional, List


class CortexEndpoint(EndpointBase):
    def create_analyzer_job(self, job: InputAnalyzerJob) -> OutputAnalyzerJob:
        return self._session.make_request(
            "POST", path="/api/connector/cortex/job", json=job
        )

    def create_responder_action(
        self, action: InputResponderAction
    ) -> OutputResponderAction:
        return self._session.make_request(
            "POST", path="/api/connector/cortex/action", json=action
        )

    def list_analyzers(self, range: Optional[str] = None) -> List[OutputAnalyzer]:
        params = {"range": range}
        return self._session.make_request(
            "GET", path="/api/connector/cortex/analyzer", params=params
        )

    def list_analyzers_by_type(self, data_type: str) -> List[OutputAnalyzer]:
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/analyzer/type/{data_type}"
        )

    def get_analyzer(self, analyzer_id: str) -> OutputAnalyzer:
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/analyzer/{analyzer_id}"
        )

    def get_analyzer_job(self, job_id: str) -> OutputAnalyzerJob:
        return self._session.make_request(
            "GET", path=f"/api/connector/cortex/job/{job_id}"
        )

    def list_responders(
        self, entity_type: str, entity_id: str
    ) -> List[OutputResponder]:
        return self._session.make_request(
            "GET", f"/api/connector/cortex/responder/{entity_type}/{entity_id}"
        )



from pathlib import Path

import pytest
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.helpers import now_to_ts
from thehive4py.types.task import OutputTask
from thehive4py.types.task_log import InputUpdateTaskLog, OutputTaskLog


class TestTaskLogEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi, test_task: OutputTask):
        created_log = thehive.task_log.create(
            task_id=test_task["_id"],
            task_log={
                "message": "My test log",
                "includeInTimeline": now_to_ts(),
                "startDate": now_to_ts(),
            },
        )

        fetched_log = thehive.task_log.get(task_log_id=created_log["_id"])

        assert created_log == fetched_log

    def test_delete(self, thehive: TheHiveApi, test_task_log: OutputTaskLog):
        thehive.task_log.delete(task_log_id=test_task_log["_id"])

        with pytest.raises(TheHiveError):
            thehive.task_log.get(task_log_id=test_task_log["_id"])

    def test_update(self, thehive: TheHiveApi, test_task_log: OutputTask):
        update_fields: InputUpdateTaskLog = {"message": "updated task log message"}
        thehive.task_log.update(task_log_id=test_task_log["_id"], fields=update_fields)

        updated_task_log = thehive.task_log.get(task_log_id=test_task_log["_id"])

        for key, value in update_fields.items():
            assert updated_task_log.get(key) == value

    def test_add_and_delete_attachment(
        self, thehive: TheHiveApi, test_task_log: OutputTaskLog, tmp_path: Path
    ):
        attachment_path = str(tmp_path / "my-attachment.txt")
        with open(attachment_path, "w") as attachment_fp:
            attachment_fp.write("some content...")

        thehive.task_log.add_attachments(
            task_log_id=test_task_log["_id"], attachment_paths=[attachment_path]
        )

        attachments = thehive.task_log.get(task_log_id=test_task_log["_id"]).get(
            "attachments", []
        )

        for attachment in attachments:
            thehive.task_log.delete_attachment(
                task_log_id=test_task_log["_id"], attachment_id=attachment["_id"]
            )

        attachments = thehive.task_log.get(task_log_id=test_task_log["_id"]).get(
            "attachments", []
        )

        assert attachments == []



from thehive4py import TheHiveApi
from thehive4py.types.case import InputCase

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

input_case: InputCase = {
    "title": "an advanced case",
    "description": "a bit more advanced case...",
    "caseTemplate": "my-template",
    "severity": 1,
    "status": "New",
    "tags": ["advanced", "example"],
}

output_case = hive.case.create(case=input_case)



from typing import List, Optional

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.organisation import (
    InputBulkOrganisationLink,
    InputOrganisation,
    InputOrganisationLink,
    InputUpdateOrganisation,
    OutputOrganisation,
    OutputSharingProfile,
)


class OrganisationEndpoint(EndpointBase):
    def create(self, organisation: InputOrganisation) -> OutputOrganisation:
        return self._session.make_request(
            "POST", path="/api/v1/organisation", json=organisation
        )

    def get(self, org_id: str) -> OutputOrganisation:
        return self._session.make_request("GET", path=f"/api/v1/organisation/{org_id}")

    def update(self, org_id: str, fields: InputUpdateOrganisation) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/organisation/{org_id}", json=fields
        )

    def delete(self, org_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/organisation/{org_id}"
        )

    def link(self, org_id: str, other_org_id: str, link: InputOrganisationLink) -> None:
        return self._session.make_request(
            "PUT", path=f"/api/v1/organisation/{org_id}/link/{other_org_id}", json=link
        )

    def unlink(self, org_id: str, other_org_id: str) -> None:
        return self._session.make_request(
            "GET", path=f"/api/v1/organisation/{org_id}/link/{other_org_id}"
        )

    def list_links(self, org_id: str) -> List[OutputOrganisation]:
        return self._session.make_request(
            "GET", path=f"/api/v1/organisation/{org_id}/links"
        )

    def bulk_link(self, org_id: str, links: List[InputBulkOrganisationLink]) -> None:
        return self._session.make_request(
            "PUT", path=f"/api/v1/organisation/{org_id}/links", json={"links": links}
        )

    def list_sharing_profiles(self) -> List[OutputSharingProfile]:
        return self._session.make_request("GET", path="/api/v1/sharingProfile")

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputOrganisation]:
        query: QueryExpr = [
            {"_name": "listOrganisation"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "organisations"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        query: QueryExpr = [
            {"_name": "listOrganisation"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "organisations.count"},
            json={"query": query},
        )



from collections import UserDict


class Paginate(UserDict):
    def __init__(self, start: int, end: int, extra_data=[]):
        super().__init__({"from": start, "to": end, "extraData": extra_data})



from thehive4py import TheHiveApi
from thehive4py.types.page import InputCasePage

hive = TheHiveApi(url="thehive.example", apikey="h1v3b33")


case_to_enrich = hive.case.create(
    case={
        "title": "case to enrich",
        "description": "a case to enrich with pages",
    }
)

case_pages: list[InputCasePage] = [
    {
        "title": "Playbooks",
        "category": "general",
        "content": "Some playbook content",
    },
    {
        "title": "Resources",
        "category": "general",
        "content": "Some useful resources",
    },
]


for case_page in case_pages:
    hive.case.create_page(case_id=case_to_enrich["_id"], page=case_page)



#!/usr/bin/env python
import argparse
import subprocess
from typing import Dict


def _run_subprocess(
    command: str,
    quiet=False,
):
    if not quiet:
        stdout = stderr = None
    else:
        stdout = stderr = subprocess.DEVNULL

    try:
        subprocess.run(str.split(command), stdout=stdout, stderr=stderr, check=True)
    except subprocess.CalledProcessError as err:
        error_output = (
            f"ERROR: Execution of command '{command}' returned: {err.returncode}\n"
        )
        print(error_output)
        exit(err.returncode)


def run_build(quiet: bool):
    print("Building thehive4py with the build module...")
    _run_subprocess(
        command="rm -rf build/ dist/",
        quiet=quiet,
    )
    _run_subprocess(
        command="python -m build --sdist --wheel",
        quiet=quiet,
    )
    print("Successfully built thehive4py!")


def run_upload(quiet: bool):
    print("Publishing thehive4py with twine...")
    _run_subprocess(
        command="twine upload dist/*",
        quiet=quiet,
    )
    print("Successfully published thehive4py!")


def run_build_docs(quiet: bool):
    print("Building thehive4py docs...")
    _run_subprocess(
        command="mkdocs build --clean --strict",
        quiet=quiet,
    )
    print("Successfully built thehive4py docs!")


def run_deploy_docs(quiet: bool):
    print("Deploying thehive4py docs to gh-pages...")
    _run_subprocess(
        command="mike deploy main latest -u -p --allow-empty",
        quiet=quiet,
    )
    print("Successfully deployed thehive4py docs to gh-pages!")


def build_run_options() -> Dict[str, dict]:
    return {
        "build": {"help": "build the package locally", "func": run_build},
        "upload": {"help": "upload the package to pypi", "func": run_upload},
        "build-docs": {"help": "build the docs locally", "func": run_build_docs},
        "deploy-docs": {"help": "deploy the docs to gh-pages", "func": run_deploy_docs},
    }


def parse_arguments(run_options: Dict[str, dict]):
    parser = argparse.ArgumentParser(
        prog="thehive4py-cd",
        description="run all cd steps or use options to run cd steps selectively",
    )
    parser.add_argument(
        "-q",
        "--quiet",
        action="store_true",
        default=False,
        help="silence verbose output",
    )

    for run_option_name, run_option_attributes in run_options.items():
        parser.add_argument(
            f"--{run_option_name.replace('_', '-')}",
            help=run_option_attributes["help"],
            action="store_true",
        )

    args = parser.parse_args()

    if not any(
        getattr(args, run_option.replace("-", "_")) for run_option in run_options
    ):
        parser.error(f"provide at least one option from: {list(run_options)}")

    return args


def main():
    run_options = build_run_options()
    args = parse_arguments(run_options=run_options)

    quiet = args.quiet

    selected_run_funcs = [
        run_option_attributes["func"]
        for run_option_name, run_option_attributes in run_options.items()
        if getattr(args, run_option_name.replace("-", "_"))
    ]

    for run_func in selected_run_funcs:
        run_func(quiet=quiet)
        print()

    print("Done!")


if __name__ == "__main__":
    main()



from thehive4py.client import TheHiveApi
from thehive4py.types.alert import OutputAlert


class TestQueryEndpoint:
    def test_simple_query(self, thehive: TheHiveApi, test_alert: OutputAlert):
        queried_alerts = thehive.query.run(
            query=[{"_name": "getAlert", "idOrName": test_alert["_id"]}],
        )

        assert [test_alert] == queried_alerts



import os.path
import tempfile
import uuid

from thehive4py import TheHiveApi
from thehive4py.types.alert import InputAlert

hive = TheHiveApi(url="http://localhost:9000", apikey="h1v3b33")

with tempfile.TemporaryDirectory() as tmpdir:

    observable_filepath = os.path.join(tmpdir, "my-observable.txt")
    with open(observable_filepath) as observable_file:
        observable_file.write("some observable content")

    attachment_key = uuid.uuid4().hex
    attachment_map = {attachment_key: observable_filepath}
    input_alert: InputAlert = {
        "type": "alert-with-file-observable",
        "source": "example",
        "sourceRef": uuid.uuid4().hex,
        "title": "alert with file observables",
        "description": "alert with file observables",
        "observables": [
            {"dataType": "file", "attachment": attachment_key},
        ],
    }

    hive.alert.create(alert=input_alert, attachment_map=attachment_map)



from typing import TypedDict


class InputProcedureRequired(TypedDict):
    occurDate: int
    patternId: str


class InputProcedure(InputProcedureRequired, total=False):
    tactic: str
    description: str


class OutputProcedureRequired(TypedDict):
    _id: str
    _createdAt: int
    _createdBy: str
    occurDate: int
    tactic: str
    tacticLabel: str
    extraData: dict


class OutputProcedure(OutputProcedureRequired, total=False):
    _updatedAt: int
    _updatedBy: str
    description: str
    patternId: str
    patternName: str


class InputUpdateProcedure(TypedDict, total=False):
    description: str
    occurDate: int



from typing import List

from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.types.task_log import InputTaskLog, InputUpdateTaskLog, OutputTaskLog


class TaskLogEndpoint(EndpointBase):
    def create(self, task_id: str, task_log: InputTaskLog) -> OutputTaskLog:
        return self._session.make_request(
            "POST", path=f"/api/v1/task/{task_id}/log", json=task_log
        )

    def get(self, task_log_id: str) -> OutputTaskLog:
        # TODO: temp implementation until a dedicated get endpoint
        logs = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getLog", "idOrName": task_log_id}]},
        )

        try:
            return logs[0]
        except IndexError:
            raise TheHiveError("404 - Task Log Not Found")

    def delete(self, task_log_id: str) -> None:
        return self._session.make_request("DELETE", path=f"/api/v1/log/{task_log_id}")

    def update(self, task_log_id: str, fields: InputUpdateTaskLog) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/log/{task_log_id}", json=fields
        )

    def add_attachments(self, task_log_id: str, attachment_paths: List[str]) -> None:
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", f"/api/v1/log/{task_log_id}/attachments", files=files
        )

    def delete_attachment(self, task_log_id: str, attachment_id: str) -> None:
        return self._session.make_request(
            "DELETE", f"/api/v1/log/{task_log_id}/attachments/{attachment_id}"
        )



from typing import Any, TypedDict


class InputCustomFieldValueRequired(TypedDict):
    name: str


class InputCustomFieldValue(InputCustomFieldValueRequired, total=False):
    value: Any
    order: int


class OutputCustomFieldValue(TypedDict):
    _id: str
    name: str
    description: str
    type: str
    value: Any
    order: int


class InputCustomFieldRequired(TypedDict):
    name: str
    group: str
    description: str
    type: str


class InputCustomField(InputCustomFieldRequired, total=False):
    displayName: str
    mandatory: bool
    options: list


class OutputCustomFieldRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    name: str
    displayName: str
    group: str
    description: str
    type: str
    mandatory: bool


class OutputCustomField(OutputCustomFieldRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    options: list


class InputUpdateCustomField(TypedDict, total=False):
    displayName: str
    group: str
    description: str
    type: str
    options: list
    mandatory: bool



from collections import UserDict as _UserDict
from typing import Any as _Any
from typing import Union as _Union
import warnings

FilterExpr = _Union["_FilterBase", dict]


class _FilterBase(_UserDict):
    """Base class for filters."""

    def __and__(self, other: "_FilterBase") -> "_FilterBase":
        if not isinstance(other, _FilterBase):
            self._raise_type_error("&", self, other)
        args = self.get("_and", [self]) + other.get("_and", [other])
        return _FilterBase(_and=args)

    def __or__(self, other: "_FilterBase") -> "_FilterBase":  # type:ignore
        if not isinstance(other, _FilterBase):
            self._raise_type_error("|", self, other)
        args = self.get("_or", [self]) + other.get("_or", [other])
        return _FilterBase(_or=args)

    def __invert__(self) -> "_FilterBase":
        return _FilterBase(_not=self)

    def _raise_type_error(self, operand, first, second):
        raise TypeError(
            f"unsupported operand type(s) for {operand}: "
            f"{type(first)} and {type(second)}"
        )


class Lt(_FilterBase):
    """Field less than value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_lt={"_field": field, "_value": value})


class Gt(_FilterBase):
    """Field greater than value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_gt={"_field": field, "_value": value})


class Lte(_FilterBase):
    """Field less than or equal value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_lte={"_field": field, "_value": value})


class Gte(_FilterBase):
    """Field less than or equal value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_gte={"_field": field, "_value": value})


class Ne(_FilterBase):
    """Field not equal value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_ne={"_field": field, "_value": value})


class Eq(_FilterBase):
    """Field equal value."""

    def __init__(self, field: str, value: _Any):
        super().__init__(_eq={"_field": field, "_value": value})


class StartsWith(_FilterBase):
    """Field starts with value."""

    def __init__(self, field: str, value: str):
        super().__init__(_startsWith={"_field": field, "_value": value})


class EndsWith(_FilterBase):
    """Field ends with value."""

    def __init__(self, field: str, value: str):
        super().__init__(_endsWith={"_field": field, "_value": value})


class Id(_FilterBase):
    """FIlter by ID."""

    def __init__(self, id: str):
        super().__init__(_id=id)


class Between(_FilterBase):
    """Field between inclusive from and exclusive to values."""

    def __init__(self, field: str, start: int, end: int):
        super().__init__(_between={"_field": field, "_from": start, "_to": end})


class In(_FilterBase):
    """Field is one of the values."""

    def __init__(self, field: _Any, values: list):
        super().__init__(_in={"_field": field, "_values": values})


class Contains(_FilterBase):
    """Object contains the field."""

    def __init__(self, field: str):
        warnings.warn(
            message="The `Contains` filter has been deprecated. "
            "Please use the `Has` filter to prevent breaking "
            "changes in the future.",
            category=DeprecationWarning,
            stacklevel=2,
        )
        super().__init__(_contains=field)


class Has(_FilterBase):
    """Object contains the field."""

    def __init__(self, field: str):
        super().__init__(_has=field)


class Like(_FilterBase):
    """Field contains the value."""

    def __init__(self, field: str, value: str):
        super().__init__(_like={"_field": field, "_value": value})


class Match(_FilterBase):
    """Field contains the value"""

    def __init__(self, field: str, value: str):
        super().__init__(_match={"_field": field, "_value": value})



from typing import List

import pytest
from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.types.case_template import InputCaseTemplate, OutputCaseTemplate


class TestCaseTemplateEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi):
        created_case_template = thehive.case_template.create(
            case_template={
                "name": "my first template",
                "description": "Template description",
            }
        )
        fetched_case_template = thehive.case_template.get(created_case_template["_id"])
        assert created_case_template == fetched_case_template

    def test_update(self, thehive: TheHiveApi, test_case_template: OutputCaseTemplate):
        case_template_id = test_case_template["_id"]
        update_fields: InputCaseTemplate = {
            "name": "updated template name",
            "description": "updated template description",
        }
        thehive.case_template.update(
            case_template_id=case_template_id, fields=update_fields
        )
        updated_case_template = thehive.case_template.get(
            case_template_id=case_template_id
        )

        for key, value in update_fields.items():
            assert updated_case_template.get(key) == value

    def test_delete(self, thehive: TheHiveApi, test_case_template: OutputCaseTemplate):
        case_template_id = test_case_template["_id"]
        thehive.case_template.delete(case_template_id=case_template_id)
        with pytest.raises(TheHiveError):
            thehive.case_template.get(case_template_id=case_template_id)

    def test_find(
        self,
        thehive: TheHiveApi,
        test_case_templates: List[OutputCaseTemplate],
    ):
        found_templates = thehive.case_template.find()
        original_ids = [template["_id"] for template in test_case_templates]
        found_ids = [template["_id"] for template in found_templates]
        assert sorted(found_ids) == sorted(original_ids)



import json
import mimetypes
import os
from typing import Optional

from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.session import TheHiveSession
from thehive4py.types.observable import InputObservable


class EndpointBase:
    def __init__(self, session: TheHiveSession):
        self._session = session

    def _fileinfo_from_filepath(self, filepath: str) -> tuple:
        filename = os.path.split(filepath)[1]
        mimetype = mimetypes.guess_type(filepath)[0]
        filestream = open(filepath, "rb")

        return (filename, filestream, mimetype)

    def _build_observable_kwargs(
        self, observable: InputObservable, observable_path: Optional[str] = None
    ) -> dict:
        if observable_path:
            kwargs = {
                "data": {"_json": json.dumps(observable)},
                "files": {"attachment": self._fileinfo_from_filepath(observable_path)},
            }
        else:
            kwargs = {"json": observable}

        return kwargs

    def _build_subquery(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> QueryExpr:
        subquery: QueryExpr = []
        if filters:
            subquery = [*subquery, {"_name": "filter", **filters}]
        if sortby:
            subquery = [*subquery, {"_name": "sort", **sortby}]
        if paginate:
            subquery = [*subquery, {"_name": "page", **paginate}]

        return subquery



from thehive4py.endpoints._base import EndpointBase
from thehive4py.errors import TheHiveError
from thehive4py.types.comment import InputComment, InputUpdateComment, OutputComment


class CommentEndpoint(EndpointBase):
    def create_in_alert(self, alert_id: str, comment: InputComment) -> OutputComment:
        return self._session.make_request(
            "POST", path=f"/api/v1/alert/{alert_id}/comment", json=comment
        )

    def create_in_case(self, case_id: str, comment: InputComment) -> OutputComment:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/comment", json=comment
        )

    def get(self, comment_id: str) -> OutputComment:
        # TODO: temp implementation until a dedicated get endpoint
        comments = self._session.make_request(
            "POST",
            path="/api/v1/query",
            json={"query": [{"_name": "getComment", "idOrName": comment_id}]},
        )
        try:
            return comments[0]
        except IndexError:
            raise TheHiveError("404 - Comment not found")

    def delete(self, comment_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/comment/{comment_id}"
        )

    def update(self, comment_id: str, fields: InputUpdateComment) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/comment/{comment_id}", json=fields
        )



from typing import List, TypedDict

from thehive4py.types.attachment import OutputAttachment


class InputObservableRequired(TypedDict):
    dataType: str


class InputObservable(InputObservableRequired, total=False):
    data: str
    message: str
    startDate: int
    tlp: int
    pap: int
    tags: List[str]
    ioc: bool
    sighted: bool
    sightedAt: int
    ignoreSimilarity: bool
    isZip: bool
    zipPassword: bool
    attachment: str


class OutputObservableRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    dataType: str
    startDate: int
    tlp: int
    pap: int
    ioc: bool
    sighted: bool
    reports: dict
    extraData: dict
    ignoreSimilarity: bool


class OutputObservable(OutputObservableRequired, total=False):
    _updatedBy: str
    _updatedAt: int
    data: str
    attachment: OutputAttachment
    tags: List[str]
    sightedAt: int
    message: str


class InputUpdateObservable(TypedDict, total=False):
    dataType: str
    message: str
    tlp: int
    pap: int
    tags: List[str]
    ioc: bool
    sighted: bool
    sightedAt: int
    ignoreSimilarity: bool


class InputBulkUpdateObservable(InputUpdateObservable):
    ids: List[str]



import datetime as dt
import time
from typing import Optional


def now_to_ts() -> int:
    """Return now as TheHive timestamp."""
    return int(time.time() * 1000)


def dt_to_ts(datetime: dt.datetime) -> int:
    """Convert datetime object to TheHive timestamp."""
    return int(datetime.timestamp() * 1000)


def ts_to_dt(timestamp: int, tz: Optional[dt.timezone] = None) -> dt.datetime:
    """Convert TheHive timestamp to datetime object."""
    return dt.datetime.fromtimestamp(timestamp / 1000.0, tz=tz)



from typing import TypedDict


class InputCasePageRequired(TypedDict):
    title: str
    content: str
    category: str


class InputCasePage(InputCasePageRequired, total=False):
    order: int


class OutputCasePageRequired(TypedDict):
    _id: str
    id: str
    createdBy: str
    createdAt: int
    title: str
    content: str
    _type: str
    slug: str
    order: int
    category: str


class OutputCasePage(OutputCasePageRequired, total=False):
    updatedBy: str
    updatedAt: int


class InputUpdateCasePage(TypedDict, total=False):
    title: str
    content: str
    category: str
    order: int



from typing import TypedDict


class OutputShareRequired(TypedDict):
    _id: str
    _type: str
    _createdBy: str
    _createdAt: int
    caseId: str
    profileName: str
    organisationName: str
    owner: bool
    taskRule: str
    observableRule: str


class OutputShare(OutputShareRequired, total=False):
    _updatedBy: str
    _updatedAt: int


class InputShareRequired(TypedDict):
    organisation: str


class InputShare(InputShareRequired, total=False):
    share: bool
    profile: str
    taskRule: str
    observableRule: str



from pathlib import Path
from typing import List

import pytest

from thehive4py.client import TheHiveApi
from thehive4py.errors import TheHiveError
from thehive4py.helpers import now_to_ts
from thehive4py.query.filters import Eq
from thehive4py.query.sort import Asc
from thehive4py.types.alert import OutputAlert
from thehive4py.types.case import (
    CaseStatus,
    ImpactStatus,
    InputBulkUpdateCase,
    InputUpdateCase,
    OutputCase,
)
from thehive4py.types.case_template import OutputCaseTemplate
from thehive4py.types.observable import InputObservable
from thehive4py.types.page import InputUpdateCasePage, OutputCasePage
from thehive4py.types.share import InputShare


class TestCaseEndpoint:
    def test_create_and_get(self, thehive: TheHiveApi):
        created_case = thehive.case.create(
            case={"title": "my first case", "description": "..."}
        )

        fetched_case = thehive.case.get(created_case["_id"])

        # TODO: sometimes timeToDetect can be different between fetched and created
        # probably it's a backend bug
        fetched_case["timeToDetect"] = created_case["timeToDetect"] = 0
        assert created_case == fetched_case

    def test_update(self, thehive: TheHiveApi, test_case: OutputCase):
        case_id = test_case["_id"]
        update_fields: InputUpdateCase = {
            "title": "my updated case",
            "description": "my updated description",
        }
        thehive.case.update(case_id=case_id, fields=update_fields)
        updated_case = thehive.case.get(case_id=case_id)

        for key, value in update_fields.items():
            assert updated_case.get(key) == value

    def test_update_with_deprecation_warning(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        case_id = test_case["_id"]
        update_fields: InputUpdateCase = {
            "title": "my updated case",
            "description": "my updated description",
        }
        with pytest.deprecated_call():
            thehive.case.update(case_id=case_id, case=update_fields)
        updated_case = thehive.case.get(case_id=case_id)

        for key, value in update_fields.items():
            assert updated_case.get(key) == value

    def test_update_with_wrong_argument_error(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        case_id = test_case["_id"]
        update_fields: InputUpdateCase = {
            "title": "my updated case",
            "description": "my updated description",
        }
        wrong_kwargs = {"case_fields": update_fields, "what": "ever"}
        with pytest.raises(TheHiveError, match=rf".*{list(wrong_kwargs.keys())}.*"):
            thehive.case.update(case_id=case_id, **wrong_kwargs)  # type:ignore

    def test_bulk_update(self, thehive: TheHiveApi, test_cases: List[OutputCase]):
        case_ids = [case["_id"] for case in test_cases]
        update_fields: InputBulkUpdateCase = {
            "ids": case_ids,
            "title": "my updated case",
            "description": "my updated description",
        }

        thehive.case.bulk_update(fields=update_fields)
        updated_cases = thehive.case.find()

        for updated_case in updated_cases:
            assert updated_case["title"] == update_fields["title"]
            assert updated_case["description"] == update_fields["description"]

    def test_merge(self, thehive: TheHiveApi, test_cases: List[OutputCase]):
        case_ids = [case["_id"] for case in test_cases]

        merged_case = thehive.case.merge(case_ids=case_ids)

        assert merged_case["title"] == " / ".join(
            [case["title"] for case in test_cases]
        )

        for case_id in case_ids:
            with pytest.raises(TheHiveError):
                thehive.case.get(case_id)

    def test_unlink_alert(
        self, thehive: TheHiveApi, test_case: OutputCase, test_alert: OutputAlert
    ):
        alert_id = test_alert["_id"]
        case_id = test_case["_id"]

        thehive.alert.merge_into_case(alert_id=alert_id, case_id=case_id)
        linked_alert = thehive.alert.get(alert_id=alert_id)
        assert linked_alert.get("caseId") == case_id

        thehive.case.unlink_alert(case_id=case_id, alert_id=alert_id)
        unlinked_alert = thehive.alert.get(alert_id=alert_id)
        assert unlinked_alert.get("caseId") is None

    def test_merge_similar_observables(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        case_id = test_case["_id"]
        thehive.case.create_observable(
            case_id=case_id, observable={"dataType": "ip", "data": "192.168.0.1"}
        )
        thehive.case.create_observable(
            case_id=case_id, observable={"dataType": "ip", "data": "192.168.0.2"}
        )

        result = thehive.case.merge_similar_observables(case_id=case_id)
        assert result == {"deleted": 0, "untouched": 2, "updated": 0}

    def test_get_linked_cases(self, thehive: TheHiveApi, test_cases: List[OutputCase]):
        common_observable: InputObservable = {
            "data": "example.com",
            "dataType": "domain",
        }

        for case in test_cases:
            thehive.case.get(case_id=case["_id"])
            thehive.case.create_observable(
                case_id=case["_id"], observable=common_observable
            )

        assert test_cases[1]["_id"] in [
            linked_case["_id"]
            for linked_case in thehive.case.get_linked_cases(
                case_id=test_cases[0]["_id"]
            )
        ]

    def test_export_and_import(
        self, thehive: TheHiveApi, test_case: OutputCase, tmp_path: Path
    ):
        archive_path = str(tmp_path / "export.thar")
        password = "test"
        thehive.case.export_to_file(
            case_id=test_case["_id"], password=password, export_path=archive_path
        )

        import_results = thehive.case.import_from_file(
            import_case={"password": password},
            import_path=archive_path,
        )
        assert import_results["case"]["title"] == test_case["title"]
        assert import_results["case"]["description"] == test_case["description"]

    def test_get_timeline(self, thehive: TheHiveApi, test_case: OutputCase):
        timeline = thehive.case.get_timeline(case_id=test_case["_id"])
        assert timeline["events"]

    def test_apply_case_template(
        self,
        thehive: TheHiveApi,
        test_case: OutputCase,
        test_case_template: OutputCaseTemplate,
    ):

        assert (
            set(test_case_template.get("tags", [])).issubset(
                set(test_case.get("tags", []))
            )
            is False
        )

        thehive.case.apply_case_template(
            fields={
                "ids": [test_case["_id"]],
                "caseTemplate": test_case_template["name"],
                "updateTags": True,
            }
        )

        updated_case = thehive.case.get(case_id=test_case["_id"])

        assert (
            set(test_case_template.get("tags", [])).issubset(
                set(updated_case.get("tags", []))
            )
            is True
        )

    def test_add_and_download_attachment(
        self, thehive: TheHiveApi, test_case: OutputCase, tmp_path: Path
    ):
        attachment_paths = [str(tmp_path / f"attachment-{i}.txt") for i in range(2)]
        download_attachment_paths = [
            str(tmp_path / f"dl-attachment-{i}.txt") for i in range(2)
        ]

        for path in attachment_paths:
            with open(path, "w") as attachment_fp:
                attachment_fp.write(f"content of {path}")

        added_attachments = thehive.case.add_attachment(
            case_id=test_case["_id"], attachment_paths=attachment_paths
        )

        for attachment, path in zip(added_attachments, download_attachment_paths):
            thehive.case.download_attachment(
                case_id=test_case["_id"],
                attachment_id=attachment["_id"],
                attachment_path=path,
            )

        for original, downloaded in zip(attachment_paths, download_attachment_paths):
            with open(original) as original_fp, open(downloaded) as downloaded_fp:
                assert original_fp.read() == downloaded_fp.read()

    def test_add_and_delete_attachment(
        self, thehive: TheHiveApi, test_case: OutputCase, tmp_path: Path
    ):
        attachment_path = str(tmp_path / "my-attachment.txt")
        with open(attachment_path, "w") as attachment_fp:
            attachment_fp.write("some content...")

        added_attachments = thehive.case.add_attachment(
            case_id=test_case["_id"], attachment_paths=[attachment_path]
        )

        for attachment in added_attachments:
            thehive.case.delete_attachment(
                case_id=test_case["_id"], attachment_id=attachment["_id"]
            )

        assert thehive.case.find_attachments(case_id=test_case["_id"]) == []

    @pytest.mark.skip(reason="organisation name is not accepted by unshare")
    def test_share_and_unshare(self, thehive: TheHiveApi, test_case: OutputCase):
        organisation = "share-org"
        share: InputShare = {"organisation": organisation}

        thehive.case.share(case_id=test_case["_id"], shares=[share])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == 1

        thehive.case.unshare(case_id=test_case["_id"], organisation_ids=[organisation])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == 1

    @pytest.mark.skip(reason="integrator container only supports a single org ")
    def test_share_and_remove_share(self, thehive: TheHiveApi, test_case: OutputCase):
        organisation = "share-org"
        share: InputShare = {"organisation": organisation}

        shares = thehive.case.share(case_id=test_case["_id"], shares=[share])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == 1

        thehive.case.remove_share(share_id=shares[0]["_id"])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == 0

    @pytest.mark.skip(reason="integrator container only supports a single org ")
    def test_share_and_set_share(self, thehive: TheHiveApi, test_case: OutputCase):
        organisation = "share-org"
        share: InputShare = {"organisation": organisation}

        shares = thehive.case.share(case_id=test_case["_id"], shares=[share])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == len(shares)

        set_shares = thehive.case.set_share(case_id=test_case["_id"], shares=[])
        assert len(thehive.case.list_shares(case_id=test_case["_id"])) == len(
            set_shares
        )

    def test_find_and_count(self, thehive: TheHiveApi, test_cases: List[OutputCase]):
        filters = Eq("title", test_cases[0]["title"]) | Eq(
            "title", test_cases[1]["title"]
        )
        found_cases = thehive.case.find(
            filters=filters,
            sortby=Asc("_createdAt"),
        )

        case_count = thehive.case.count(filters=filters)

        assert found_cases == test_cases
        assert len(test_cases) == case_count

    def test_delete(self, thehive: TheHiveApi, test_case: OutputCase):
        case_id = test_case["_id"]
        thehive.case.delete(case_id)
        with pytest.raises(TheHiveError):
            thehive.case.get(case_id)

    def test_create_and_get_observable(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        created_observables = thehive.case.create_observable(
            test_case["_id"], {"dataType": "domain", "data": "example.com"}
        )
        case_observables = thehive.case.find_observables(test_case["_id"])
        assert created_observables == case_observables

    def test_create_observable_from_file(
        self, thehive: TheHiveApi, test_case: OutputCase, tmp_path: Path
    ):
        observable_path = str(tmp_path / "case-observable.txt")
        with open(observable_path, "w") as observable_fp:
            observable_fp.write("observable content")

        created_observable = thehive.observable.create_in_case(
            case_id=test_case["_id"],
            observable={
                "dataType": "file",
                "message": "file based observable",
            },
            observable_path=observable_path,
        )[0]

        fetched_observable = thehive.observable.get(
            observable_id=created_observable["_id"]
        )
        assert created_observable == fetched_observable

        attachment = fetched_observable.get("attachment")
        assert attachment and attachment["name"] in observable_path

    def test_create_and_get_task(self, thehive: TheHiveApi, test_case: OutputCase):
        created_task = thehive.case.create_task(
            case_id=test_case["_id"], task={"title": "my task"}
        )
        case_tasks = thehive.case.find_tasks(case_id=test_case["_id"])
        assert created_task in case_tasks

    def test_close_and_open(self, thehive: TheHiveApi, test_case: OutputCase):
        case_id = test_case["_id"]
        assert test_case["status"] == "New"
        close_status = CaseStatus.TruePositive
        close_impact = ImpactStatus.WithImpact
        close_summary = "Closed..."
        thehive.case.close(
            case_id=case_id,
            status=close_status,
            summary=close_summary,
            impact_status=close_impact,
        )
        closed_case = thehive.case.get(case_id)
        assert closed_case["status"] == close_status
        assert closed_case.get("impactStatus") == close_impact
        assert closed_case.get("summary") == close_summary

        open_status = CaseStatus.InProgress
        thehive.case.open(case_id, status=open_status)
        reopened_case = thehive.case.get(case_id)
        assert reopened_case["status"] == open_status

    def test_find_comments(self, thehive: TheHiveApi, test_case: OutputCase):
        created_comment = thehive.comment.create_in_case(
            case_id=test_case["_id"],
            comment={"message": "my first comment"},
        )

        case_comments = thehive.case.find_comments(case_id=test_case["_id"])

        assert [created_comment] == case_comments

    def test_create_and_find_procedure(
        self, thehive: TheHiveApi, test_case: OutputCase
    ):
        created_procedure = thehive.case.create_procedure(
            case_id=test_case["_id"],
            procedure={
                "occurDate": now_to_ts(),
                "patternId": "T1059.007",
                "tactic": "execution",
                "description": "...",
            },
        )
        case_procedures = thehive.case.find_procedures(case_id=test_case["_id"])
        assert [created_procedure] == case_procedures

    def test_create_and_find_page(self, thehive: TheHiveApi, test_case: OutputCase):
        created_page = thehive.case.create_page(
            case_id=test_case["_id"],
            page={"title": "my case page", "category": "testing", "content": "..."},
        )

        case_pages = thehive.case.find_pages(case_id=test_case["_id"])
        assert [created_page] == case_pages

    def test_update_and_delete_page(
        self, thehive: TheHiveApi, test_case: OutputCase, test_case_page: OutputCasePage
    ):
        update_page: InputUpdateCasePage = {"title": "my updated case page"}
        thehive.case.update_page(
            case_id=test_case["_id"],
            page_id=test_case_page["_id"],
            page=update_page,
        )

        updated_case_page = thehive.case.find_pages(
            case_id=test_case["_id"], filters=Eq("_id", test_case_page["_id"])
        )[0]

        for key, value in update_page.items():
            assert updated_case_page.get(key) == value

        thehive.case.delete_page(
            case_id=test_case["_id"], page_id=test_case_page["_id"]
        )
        assert len(thehive.case.find_pages(case_id=test_case["_id"])) == 0



from typing import TypedDict


class InputComment(TypedDict):
    message: str


class OutputCommentRequired(TypedDict):
    _id: str
    _type: str
    createdBy: str
    createdAt: int
    message: str
    isEdited: bool


class OutputComment(OutputCommentRequired, total=False):
    updatedAt: str


class InputUpdateComment(TypedDict):
    message: str



from thehive4py.client import TheHiveApi
from thehive4py.helpers import now_to_ts
from thehive4py.types.case import OutputCase
from thehive4py.types.timeline import InputUpdateCustomEvent, OutputCustomEvent


class TestTimelineEndpoint:
    def test_get(self, thehive: TheHiveApi, test_case: OutputCase):
        timeline = thehive.timeline.get(case_id=test_case["_id"])
        assert timeline["events"]

    def test_create_and_delete_event(self, thehive: TheHiveApi, test_case: OutputCase):
        timeline_event = thehive.timeline.create_event(
            case_id=test_case["_id"],
            event={"date": now_to_ts(), "title": "custom timeline event"},
        )

        timeline_events = thehive.timeline.get(case_id=test_case["_id"])["events"]
        assert timeline_event["_id"] in [event["entityId"] for event in timeline_events]

        thehive.timeline.delete_event(event_id=timeline_event["_id"])
        timeline_events = thehive.timeline.get(case_id=test_case["_id"])["events"]
        assert timeline_event["_id"] not in [
            event["entityId"] for event in timeline_events
        ]

    def test_update_event(
        self, thehive: TheHiveApi, test_timeline_event: OutputCustomEvent
    ):
        event_id = test_timeline_event["_id"]
        update_fields: InputUpdateCustomEvent = {
            "date": now_to_ts(),
            "endDate": now_to_ts(),
            "title": "updated event",
            "description": "updated event description",
        }
        thehive.timeline.update_event(event_id=event_id, fields=update_fields)



import json as jsonlib
from collections import UserDict
from os import PathLike
from typing import Any, Optional, Union

import requests
import requests.adapters
import requests.auth
from urllib3 import Retry

from thehive4py.__version__ import __version__
from thehive4py.errors import TheHiveError

DEFAULT_RETRY = Retry(
    total=5,
    backoff_factor=1,
    status_forcelist=[500, 502, 503, 504],
    allowed_methods=["GET", "POST", "PUT", "PATCH", "DELETE"],
    raise_on_status=False,
)


RetryValue = Union[Retry, int, None]
VerifyValue = Union[bool, str]


class _SessionJSONEncoder(jsonlib.JSONEncoder):
    """Custom JSON encoder class for TheHive session."""

    def default(self, o: Any):
        if isinstance(o, UserDict):
            return o.data
        return super().default(o)


class TheHiveSession(requests.Session):
    def __init__(
        self,
        url: str,
        apikey: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        verify: VerifyValue = True,
        max_retries: RetryValue = DEFAULT_RETRY,
    ):
        super().__init__()
        self.hive_url = self._sanitize_hive_url(url)
        self.verify = verify
        self.headers["User-Agent"] = f"thehive4py/{__version__}"
        self._set_retries(max_retries=max_retries)

        if username and password:
            self.headers["Authorization"] = requests.auth._basic_auth_str(
                username, password
            )
        elif apikey:
            self.headers["Authorization"] = f"Bearer {apikey}"
        else:
            raise TheHiveError(
                "Either apikey or the username/password combination must be provided!"
            )

    def _set_retries(self, max_retries: RetryValue):
        """Configure the session to retry."""
        retry_adapter = requests.adapters.HTTPAdapter(max_retries=max_retries)
        self.mount("http://", retry_adapter)
        self.mount("https://", retry_adapter)

    def _sanitize_hive_url(self, hive_url: str) -> str:
        """Sanitize the base url for the client."""
        if hive_url.endswith("/"):
            return hive_url[:-1]
        return hive_url

    def make_request(
        self,
        method: str,
        path: str,
        params=None,
        data=None,
        json=None,
        files=None,
        download_path: Union[str, PathLike, None] = None,
    ) -> Any:
        endpoint_url = f"{self.hive_url}{path}"

        headers = {**self.headers}
        if json:
            data = jsonlib.dumps(json, cls=_SessionJSONEncoder)
            headers = {**headers, "Content-Type": "application/json"}

        response = self.request(
            method,
            url=endpoint_url,
            params=params,
            data=data,
            files=files,
            headers=headers,
            verify=self.verify,
            stream=bool(download_path),
        )

        return self._process_response(response, download_path=download_path)

    def _process_response(
        self,
        response: requests.Response,
        download_path: Union[str, PathLike, None] = None,
    ):
        if response.ok:
            if download_path is None:
                return self._process_text_response(response)
            else:
                self._process_stream_response(
                    response=response, download_path=download_path
                )

        if not response.ok:
            self._process_error_response(response=response)

    def _process_text_response(self, response: requests.Response):
        try:
            json_data = response.json()
        except requests.exceptions.JSONDecodeError:
            json_data = None

        if json_data is None:
            return response.text
        return json_data

    def _process_stream_response(
        self, response: requests.Response, download_path: Union[str, PathLike]
    ):
        with open(download_path, "wb") as download_fp:
            for chunk in response.iter_content(chunk_size=4096):
                download_fp.write(chunk)

    def _process_error_response(self, response: requests.Response):
        try:
            json_data = response.json()
        except requests.exceptions.JSONDecodeError:
            json_data = None

        if isinstance(json_data, dict) and all(
            [
                "type" in json_data,
                "message" in json_data,
            ]
        ):
            error_text = f"{json_data['type']} - {json_data['message']}"
        else:
            error_text = response.text
        raise TheHiveError(message=error_text, response=response)



#!/usr/bin/env python
import argparse
import subprocess
from typing import List


def _run_subprocess(
    command: str,
    quiet=False,
):
    if not quiet:
        stdout = stderr = None
    else:
        stdout = stderr = subprocess.DEVNULL

    try:
        subprocess.run(str.split(command), stdout=stdout, stderr=stderr, check=True)
    except subprocess.CalledProcessError as err:
        error_output = (
            f"ERROR: Execution of command '{command}' returned: {err.returncode}\n"
        )
        print(error_output)
        exit(err.returncode)


def check_all(quiet=False):
    print("Run all checks...")
    check_lint(quiet=quiet)
    check_format(quiet=quiet)
    check_type(quiet=quiet)
    check_cve(quiet=quiet)
    check_security(quiet=quiet)
    print("All checks succeeded!")


def check_lint(quiet=False):
    print("Run lint checks with flake8...")
    _run_subprocess(
        command="flake8 thehive4py/ tests/",
        quiet=quiet,
    )
    print("Lint checks succeeded!")


def check_format(quiet=False):
    print("Run format checks with black...")
    _run_subprocess(
        command="black --check thehive4py/ tests/",
        quiet=quiet,
    )
    print("Format checks succeeded!")


def check_type(quiet=False):
    print("Run type checks with mypy...")
    _run_subprocess(
        command="mypy --install-types --non-interactive thehive4py/",
        quiet=quiet,
    )
    print("Type checks succeeded!")


def check_cve(quiet=False):
    print("Run CVE checks with pip-audit...")
    _run_subprocess(
        command="pip-audit .",
        quiet=quiet,
    )
    print("CVE checks succeeded!")


def check_security(quiet=False):
    print("Run security checks with bandit...")
    _run_subprocess(
        command="bandit -r thehive4py/",
        quiet=quiet,
    )
    print("Security checks succeeded!")


def check_test(quiet=False):
    print("Run integration tests with pytest...")
    _run_subprocess(
        command="pytest -v --cov",
        quiet=quiet,
    )
    print("Integration tests succeeded!")


def build_check_options() -> List[dict]:
    return [
        {"name": "lint", "help": "run lint checks", "check": check_lint},
        {"name": "format", "help": "run format checks", "check": check_format},
        {"name": "type", "help": "run type checks", "check": check_type},
        {"name": "cve", "help": "run cve checks", "check": check_cve},
        {"name": "security", "help": "run security checks", "check": check_security},
        {"name": "test", "help": "run integration tests", "check": check_test},
    ]


def parse_arguments(check_options: List[dict]):
    parser = argparse.ArgumentParser(
        prog="thehive4py-ci",
        description=(
            "run all ci checks except tests by default, "
            "use options to run ci checks selectively"
        ),
    )
    parser.add_argument(
        "-q",
        "--quiet",
        action="store_true",
        default=False,
        help="silence verbose output",
    )
    for check_option in check_options:
        parser.add_argument(
            f"--{check_option['name']}",
            help=check_option["help"],
            action="store_true",
        )

    return parser.parse_args()


def main():
    check_options = build_check_options()
    args = parse_arguments(check_options=check_options)

    quiet = args.quiet

    selective_checks = [
        check_option["check"]
        for check_option in check_options
        if getattr(args, check_option["name"])
    ]

    if selective_checks:
        for check in selective_checks:
            check(quiet=quiet)
    else:
        check_all(quiet=quiet)


if __name__ == "__main__":
    main()
